#include "apishared.h"

 
/* Read a line of input - read at most max-1 characters, stopping
   if \n is encountered (if \n is encountered then it is entered
   into the array then converted to \0). 
   The characters read are terminated with \0.
*/
int getline(char *line, int max) 
{
	if (fgets(line, max, stdin) == NULL)
		return 0;
	else {

		/* Check for a \n - if find one then change it to \0 */
		convert_n_to_0(line);
		return strlen(line);
	}
}



/* If find a \n then change it to \0 */
void convert_n_to_0(char *line)
{
        while (*line != '\0') {
                if (*line == '\n') {
                        *line = '\0';
                        return;
                }
                line++;
        }
}
 

/* Post scanf check and clean up */
int post_scanf_check(int result)
{
        char garbage[1000];
 
        /* If scanf conversion conflicts with input then
           scanf returns 0 and the offending character is left in the
           input stream (fflush(stdin) doesnt remove it) */
        if (result == 0)
                scanf("%s", garbage);
	
	/* If scanf conversion terminates with EOF (CNTRL+D) then
    	   exit */
	if (result == EOF) {
		printf("\nReceived end of file - exiting\n");
		exit(1);
	}
}



int post_gets_check(char *result)
{
	/* If EOF (CNTRL+D) or error occurred during gets then
    	   exit */
	if (result == NULL) {
		printf("\nReceived end of file/error during gets - exiting in five seconds\n");
		sleep(5);
		exit(1);
	}
}

/* Routines to print out a DbvProtocolAndSyntaxes structure */
int PrintApplicationProtocol(DbvApplicationProtocol *peApplicationProtocol)
{
        printf("peApplicationProtocol == ");
        switch(*peApplicationProtocol)
        {
        case Unknown_Protocol:
                printf("Unknown_Protocol\n");
                break;
        case Unidentified_Protocol:
                printf("Unidentified_Protocol\n");
                break;
        case SR_Protocol:
                printf("SR_Protocol\n");
                break;
        case Z3950_Protocol:
                printf("Z3950_Protocol\n");
                break;
        default:
                printf("Error in value!!\n");
                break;
        }
}
 
int PrintAbstractSyntax(DbvAbstractSyntax *peAbstractSyntax)
{
        printf("peAbstractSyntax == ");
        switch(*peAbstractSyntax)
        {
        case No_Syntaxes_Provided:
                printf("No_Syntaxes_Provided\n");
                break;
        case No_Common_Syntax:
                printf("No_Common_Syntax\n");
                break;
        case Common_Syntaxes:
                printf("Common_Syntaxes\n");
                break;
        default:
                printf("Error in value!!\n");
                break;
        }
}
 
int PrintppcCommonSyntaxes(char **ppcCommonSyntaxes)
{
        int lLoop=0;
        printf("ppcCommonSyntaxes ==\n");
        if(ppcCommonSyntaxes == (char **)NULL)
                printf("NULL\n");
        else
                while(ppcCommonSyntaxes[lLoop] != NULL)
        {
                printf("\t%s\n", ppcCommonSyntaxes[lLoop]);
                lLoop++;
        }
 
}









int	fill_dbv_sutrs_external(DbvASN1External *external, char *sutrs)
{


	char	*oid = ANSI_UNSTRUCTURED_TEXT_RECORD;

	/* ALWAYS SUPPLY AN OID WITH SINGLE ASN1 TYPE */

	/* +1 to copy the \0 */
	external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
	memcpy(external->pcObjectIdentifier, ANSI_UNSTRUCTURED_TEXT_RECORD, strlen(oid));

	external->eValueType = EVT_SingleASN1Type;
	external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
	external->uExternalValue.psSingleASN1Type->eASN1Type = AT_UnstructuredText;
	external->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText = (struct DbvInternationalString *) DbvAlloc (sizeof (struct DbvInternationalString ));

	fill_dbv_inter_string(external->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText, sutrs);

}



int	fill_dbv_isosutrs_external(DbvASN1External *external, char *sutrs)
{
	char	*oid = ISO_UNSTRUCTURED_TEXT_RECORD;
	char	sutrs_string[100];

	strcpy(sutrs_string, sutrs);

	/* ALWAYS SUPPLY AN OID WITH SINGLE ASN1 TYPE */

	/* +1 to copy the \0 */
	external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
	memcpy(external->pcObjectIdentifier, oid, strlen(oid));

	external->eValueType = EVT_SingleASN1Type;
	external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
	external->uExternalValue.psSingleASN1Type->eASN1Type = AT_UnstructuredText;
	external->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText = (struct DbvInternationalString *) DbvAlloc (sizeof (struct DbvInternationalString ));
	fill_dbv_inter_string(external->uExternalValue.psSingleASN1Type->uASN1.psUnstructuredText, sutrs_string);
}




/* There isnt a general function to set up OPAC records - they are set
up completely as required  */


int fill_dbv_summary_external(DbvASN1External *external, char *title, char *author, char *callnum, char *rectype, char *biblevel, DbvFormatSpec **myformat, char *pubplace, char *pubdate, char *targetkey, char *satelmnt, unsigned long *rank, char *docid, char *abstract, DbvOtherInformation *myother)
{
        char *oid = ANSI_SUMMARY_RECORD;
 
        /* ALWAYS SUPPLY AN OID WITH SINGLE ASN1 TYPE */
        /* +1 to copy the \0 */
        external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
        memcpy(external->pcObjectIdentifier, oid, strlen(oid));

 
        external->eValueType = EVT_SingleASN1Type;
        external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
        external->uExternalValue.psSingleASN1Type->eASN1Type = AT_SummaryRecord;
        external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord = (struct DbvSummaryRecord *) DbvAlloc (sizeof (struct DbvSummaryRecord));


	fill_dbv_inter_string(&external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord->sTitle, title);


	fill_summary_bulk(external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord, title, author, callnum, rectype, biblevel, pubplace, pubdate, targetkey, satelmnt, rank, docid, abstract);

	/* Now assign pointers passed down as parameters  - provided
	   the contents of what they point to arn't changed or lost
	   everything will be fine */
	if (myformat != NULL)	
		external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord->ppsFormat =  myformat;


	if (myother != NULL) {

		/* Have an otherinformation list with one node:  0, \0 */
		external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof(DbvOtherInformation *) * 2);
		external->uExternalValue.psSingleASN1Type->uASN1.psSummaryRecord->ppsOtherInformation[0] = myother;
	}

}


int fill_summary_bulk(DbvSummaryRecord *psSummaryRecord, char *title, char *author, char *callnum, char *rectype, char *biblevel, char *pubplace, char *pubdate, char *targetkey, char *satelmnt, unsigned long *rank, char *docid, char *abstract)
{
	/* stitle is a compulsory field */
	fill_dbv_inter_string(&psSummaryRecord->sTitle, title);

	/* If psAuthor is given (it is an optional field) */
	if (author != NULL) {
		psSummaryRecord->psAuthor = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psAuthor, author);
	}

	/* If callnum is given (it is an optional field) */
	if (callnum != NULL) {
		psSummaryRecord->psCallNumber = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psCallNumber, callnum);
	}

	/* If rectype is given (it is an optional field) */
	if (rectype != NULL) {
		psSummaryRecord->psRecordType = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psRecordType, rectype);
	}

	/* If biblevel is given (it is an optional field) */
	if (biblevel != NULL) {
		psSummaryRecord->psBibliographicLevel = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psBibliographicLevel, biblevel);
	}

	/* If pubplace is given (it is an optional field) */
	if (pubplace != NULL) {
		psSummaryRecord->psPublicationPlace = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psPublicationPlace, pubplace);
	}

	/* If pubdate is given (it is an optional field) */
	if (pubdate != NULL) {
		psSummaryRecord->psPublicationDate = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psPublicationDate, pubdate);
	}

	/* If targetkey is given (it is an optional field) */
	if (targetkey != NULL) {
		psSummaryRecord->psTargetSystemKey = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psTargetSystemKey, targetkey);
	}

	/* If satelmnt is given (it is an optional field) */
	if (satelmnt != NULL) {
		psSummaryRecord->psSatisfyingElement = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psSatisfyingElement, satelmnt);
	}

	/* If rank is given (it is an optional field) */
	if (rank != NULL) {
		psSummaryRecord->pulRank = (Ulong *) DbvAlloc (sizeof(Ulong));
		*psSummaryRecord->pulRank = *rank;
	}

	/* If docid is given (it is an optional field) */
	if (docid != NULL) {
		psSummaryRecord->psDocumentId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psDocumentId, docid);
	}

	/* If abstract is given (it is an optional field) */
	if (abstract != NULL) {
		psSummaryRecord->psAbstract = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psSummaryRecord->psAbstract, abstract);
	}


}



int	fill_dbv_aligned_external(DbvASN1External *external, char *aligned, char *oid)
{
	char	*string = NULL;
	string = aligned;

	/* If an oid exists */
	if (oid != NULL) {

		/* +1 to copy the \0 */
		external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
		memcpy(external->pcObjectIdentifier, oid, strlen(oid));
	}


	external->eValueType = EVT_OctetAligned;
	external->uExternalValue.psOctetAligned = (DbvOctetString * ) DbvAlloc (sizeof (DbvOctetString));

	fill_dbv_octet_string(external->uExternalValue.psOctetAligned, string);
}



int	fill_dbv_arbitrary_external(DbvASN1External *external, char *arbitrary, char *oid)
{
	char	*string = NULL;
	string = arbitrary;

	/* If an oid exists */
	if (oid != NULL) {

		/* +1 to copy the \0 */
		external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
		memcpy(external->pcObjectIdentifier, oid, strlen(oid));
	}


	external->eValueType = EVT_Arbitrary;


	/* Arbirary is a bit string */
	external->uExternalValue.psArbitrary = (DbvOctetString * ) DbvAlloc (sizeof (DbvOctetString));

	fill_dbv_octet_string(external->uExternalValue.psArbitrary, string);
}






int	fill_dbv_otherinfo_char(DbvOtherInformation *psOtherInformation, DbvBool want_infocat, char *oid, unsigned long catvalue,
 char *charinfo)
{


	/* inforcategory is optional */

	if (want_infocat == DBV_TRUE) {
		psOtherInformation->psInfoCategory = (DbvInfoCategory * ) DbvAlloc (sizeof (DbvInfoCategory));

		/* pcCategoryTypeId is ObjectIdentifier* which is char* */

		/* +1 to copy the \0 */

		if (oid != NULL) {
			psOtherInformation->psInfoCategory->pcCategoryTypeId = (char *) DbvAlloc (strlen(oid) + 1);
			memcpy(psOtherInformation->psInfoCategory->pcCategoryTypeId, oid, strlen(oid));
		}
		psOtherInformation->psInfoCategory->ulCategoryValue = catvalue;
	}

	psOtherInformation->eInformationType = IT_CharacterInfo;
	psOtherInformation->uInformation.psCharacterInfo = (DbvInternationalString * ) DbvAlloc (sizeof (DbvInternationalString));
	fill_dbv_inter_string(psOtherInformation->uInformation.psCharacterInfo, charinfo);


}



int	fill_dbv_otherinfo_bin(DbvOtherInformation *psOtherInformation, DbvBool want_infocat, char *oid, unsigned long catvalue,
 char *bininfo)
{


	/* inforcategory is optional */

	if (want_infocat == DBV_TRUE) {
		psOtherInformation->psInfoCategory = (DbvInfoCategory * ) DbvAlloc (sizeof (DbvInfoCategory));

		/* pcCategoryTypeId is ObjectIdentifier* which is char* */

		/* +1 to copy the \0 */

		if (oid != NULL) {
			psOtherInformation->psInfoCategory->pcCategoryTypeId = (char *) DbvAlloc (strlen(oid) + 1);
			memcpy(psOtherInformation->psInfoCategory->pcCategoryTypeId, oid, strlen(oid));
		}

		psOtherInformation->psInfoCategory->ulCategoryValue = catvalue;
	}


	psOtherInformation->eInformationType = IT_BinaryInfo;
	psOtherInformation->uInformation.psBinaryInfo = (DbvOctetString * ) DbvAlloc (sizeof (DbvOctetString));
	fill_dbv_inter_string(psOtherInformation->uInformation.psCharacterInfo, bininfo);
}



int	fill_dbv_otherinfo_ext(DbvOtherInformation *psOtherInformation, DbvBool want_infocat, char *oid, unsigned long catvalue)
{


	/* inforcategory is optional */

	if (want_infocat == DBV_TRUE) {
		psOtherInformation->psInfoCategory = (DbvInfoCategory * ) DbvAlloc (sizeof (DbvInfoCategory));

		/* pcCategoryTypeId is ObjectIdentifier* which is char* */

		/* +1 to copy the \0 */

		if (oid != NULL) {
			psOtherInformation->psInfoCategory->pcCategoryTypeId = (char *) DbvAlloc (strlen(oid) + 1);
			memcpy(psOtherInformation->psInfoCategory->pcCategoryTypeId, oid, strlen(oid));
		}

		psOtherInformation->psInfoCategory->ulCategoryValue = catvalue;
	}

	psOtherInformation->eInformationType = IT_ExternalInfo;
	psOtherInformation->uInformation.psExternallyDefinedInfo = (DbvASN1External * ) DbvAlloc (sizeof (DbvASN1External));

}


int	fill_dbv_otherinfo_obj(DbvOtherInformation *psOtherInformation, DbvBool want_infocat, char *infooid, unsigned long catvalue,
 char *otheroid)
{


	/* inforcategory is optional */

	if (want_infocat == DBV_TRUE) {
		psOtherInformation->psInfoCategory = (DbvInfoCategory * ) DbvAlloc (sizeof (DbvInfoCategory));

		/* pcCategoryTypeId is ObjectIdentifier* which is char* */

		/* +1 to copy the \0 */

		if (infooid != NULL) {
			psOtherInformation->psInfoCategory->pcCategoryTypeId = (char *) DbvAlloc (strlen(infooid) + 1);
			memcpy(psOtherInformation->psInfoCategory->pcCategoryTypeId, infooid, strlen(infooid));
		}

		psOtherInformation->psInfoCategory->ulCategoryValue = catvalue;
	}

	psOtherInformation->eInformationType = IT_ObjectIdentifier;

	/* Set up an object identifier */
	/* +1 to copy the \0 */
	psOtherInformation->uInformation.pcObjectIdentifier = (char *) DbvAlloc (strlen(otheroid) + 1);
	memcpy(psOtherInformation->uInformation.pcObjectIdentifier, otheroid, strlen(otheroid));

}



void fill_DbvOctetString(DbvOctetString *octet_string, char *setup_string)
{
	fill_dbv_octet_string(octet_string, setup_string);
}


int fill_dbv_octet_string(DbvOctetString *octet_string, char *setup_string)
{
	octet_string->ulLength = (strlen(setup_string));
	octet_string->pcValue = (char *) DbvAlloc (strlen(setup_string));
	memcpy(octet_string->pcValue, setup_string, strlen(setup_string));
}



int fill_dbv_inter_string(DbvInternationalString *inter_string, char *setup_string) 
{

	inter_string->ulLength = (strlen(setup_string));
	inter_string->pcValue = (char *) DbvAlloc (strlen(setup_string));
	memcpy(inter_string->pcValue, setup_string, strlen(setup_string));
}



int fill_DbvGeneralString(DbvGeneralString *gen_string, char *setup_string) 
{
	gen_string->ulLength = (strlen(setup_string));
	gen_string->pcValue = (char *) DbvAlloc (strlen(setup_string));
	memcpy(gen_string->pcValue, setup_string, strlen(setup_string));
}






int fill_dbv_nameplus_rec(DbvNamePlusRecord *psResponseRecord, char *dbname, DbvASN1External* external)
{

	if (dbname != NULL) {
		psResponseRecord->psDatabaseName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		fill_dbv_inter_string(psResponseRecord->psDatabaseName, dbname); 
	}

	psResponseRecord->eDbRecordType = DRT_Retrieval;
	psResponseRecord->uDbRecord.psRetrievalRecord = external;
}



int fill_dbv_nrec_diagn_default(DbvNamePlusRecord *psResponseRecord, char *diag_dbname, char *oid, unsigned long condition, char *diag, int choice, DbvDiagnosticsType diagtype, DbvASN1External *external)
{
	if (diag_dbname != NULL) {
		psResponseRecord->psDatabaseName = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		fill_dbv_inter_string(psResponseRecord->psDatabaseName, diag_dbname); 
	}

	psResponseRecord->eDbRecordType = DRT_SurrogateDiagnostics;
	psResponseRecord->uDbRecord.psSurrogateDiagnostic = (DbvDiagRec *) DbvAlloc (sizeof (DbvDiagRec));
	fill_default_diag_rec(psResponseRecord->uDbRecord.psSurrogateDiagnostic, oid, condition, diag, choice, diagtype, external);
	
}



int fill_default_diag_rec(DbvDiagRec *psSurrogateDiagnostic, char *oid, unsigned long condition, char *diag, int choice, DbvDiagnosticsType diagtype, DbvASN1External *external)
{

	switch(diagtype) {
	case DT_DefaultFormat:
		psSurrogateDiagnostic->eDiagnosticsType = DT_DefaultFormat;

		psSurrogateDiagnostic->uDiagnostics.psDefault = (DbvDefaultDiagFormat *) DbvAlloc (sizeof(DbvDefaultDiagFormat));
		switch (choice) {
		case PVIF_Version3:
			fill_default_diag_format_v3(psSurrogateDiagnostic->uDiagnostics.psDefault, oid, condition, diag);
			break;
		case PVIF_Version2:
			fill_default_diag_format_v2(psSurrogateDiagnostic->uDiagnostics.psDefault, oid, condition, diag);
			break;
		}
		break;


	case DT_ExternallyDefined:
		psSurrogateDiagnostic->eDiagnosticsType = DT_ExternallyDefined;
		psSurrogateDiagnostic->uDiagnostics.psExternallyDefined = external;
		break;
	}
}



int fill_default_diag_format_v2(DbvDefaultDiagFormat *psDefault, char *oid, unsigned long condition, char *ver2diag)
{
	psDefault->pcDiagnosticSetId = (char *) DbvAlloc (strlen(oid) + 1);
	memcpy(psDefault->pcDiagnosticSetId, oid, strlen(oid));
	psDefault->ulCondition = condition;

	psDefault->sAdditionalInfo.eProtocolVersionInForce = PVIF_Version2;
	psDefault->sAdditionalInfo.uAdditionalInfo.pcV2 = (DbvVisibleString) DbvAlloc (strlen(ver2diag) + 1);
	memcpy(psDefault->sAdditionalInfo.uAdditionalInfo.pcV2, ver2diag, strlen(ver2diag));
}



int fill_default_diag_format_v3(DbvDefaultDiagFormat *psDefault, char *oid, unsigned long condition, char *ver3diag)
{
	psDefault->pcDiagnosticSetId = (char *) DbvAlloc (strlen(oid) + 1);
	memcpy(psDefault->pcDiagnosticSetId, oid, strlen(oid));
	psDefault->ulCondition = condition;

	psDefault->sAdditionalInfo.eProtocolVersionInForce = PVIF_Version3;
	psDefault->sAdditionalInfo.uAdditionalInfo.psV3 = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
	fill_dbv_inter_string(psDefault->sAdditionalInfo.uAdditionalInfo.psV3, ver3diag); 
	
}







void fill_DbvStringOrNumeric(DbvStringOrNumeric *psUnitType, DbvStringOrNumericType eStringOrNumeric)
{
	 fill_strornum(psUnitType, eStringOrNumeric);
}



int fill_strornum(DbvStringOrNumeric *psUnitType, DbvStringOrNumericType eStringOrNumeric)
{
        char *test_string = "string or numeric test string";
        unsigned long test_numeric = 9999;
 
        switch (eStringOrNumeric) {
        case SNT_String:

                psUnitType->eStringOrNumeric = SNT_String;
                psUnitType->uStringOrNumeric.psString = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
                fill_dbv_inter_string(psUnitType->uStringOrNumeric.psString, test_string);
                break;
        case SNT_Numeric:
 
                psUnitType->eStringOrNumeric = SNT_Numeric;
                psUnitType->uStringOrNumeric.ulNumeric = test_numeric;
                break;
        }
 
}



void fill_DbvIntUnit(DbvIntUnit *psIntegerAndUnit)
{
	fill_intunit(psIntegerAndUnit);
}


int fill_intunit(DbvIntUnit *psIntegerAndUnit)
{
        psIntegerAndUnit->ulValue = 55;
        fill_unit(&psIntegerAndUnit->sUnitUsed);
}
 
 

void fill_DbvUnit(DbvUnit *psUnitUsed)
{
	fill_unit(psUnitUsed);
}



int fill_unit(DbvUnit *psUnitUsed)
{
        char *test_unit_system = "test unit system";
 
        /* All these are optional */
 
        psUnitUsed->psUnitSystem = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
        fill_dbv_inter_string(psUnitUsed->psUnitSystem, test_unit_system);
 
 
        psUnitUsed->psUnitType = (DbvStringOrNumeric *) DbvAlloc (sizeof(DbvStringOrNumeric));
        fill_strornum(psUnitUsed->psUnitType, SNT_String);
 
 
        psUnitUsed->psUnit = (DbvStringOrNumeric *) DbvAlloc (sizeof(DbvStringOrNumeric));
        fill_strornum(psUnitUsed->psUnit, SNT_Numeric);
 
        psUnitUsed->pulScaleFactor = (Ulong *) DbvAlloc (sizeof(Ulong));
        *psUnitUsed->pulScaleFactor = 56;
 
}
 
 




int fill_attsplusterm(DbvAttributesPlusTerm *psAttributesPlusTerm)
{

	printf("Attributes:\n");

	/* ppsAttributeElement */

	/* Create three list nodes 0, 1, \0 */
	psAttributesPlusTerm->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc (sizeof(DbvAttributeElement *) * 3);

	/* First node */
	psAttributesPlusTerm->ppsAttributeElement[0] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
	printf("Node 0: ");
	fill_attelement(psAttributesPlusTerm->ppsAttributeElement[0]);

	/* Second node */
	psAttributesPlusTerm->ppsAttributeElement[1] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
	printf("Node 1: ");
	fill_attelement(psAttributesPlusTerm->ppsAttributeElement[1]);

	/* NULL list terminator */
	psAttributesPlusTerm->ppsAttributeElement[2] = NULL;


	printf("Term:\n");

	/* sTerm */
	fill_term(&psAttributesPlusTerm->sTerm, TT_Numeric);
}



int fill_attelement(DbvAttributeElement *psAttributeElement)
{
	/* A randomly chosen optional oid */
	psAttributeElement->pcAttributeSetId = ANSI_MAB;

	psAttributeElement->ulAttributeType = 44;
	fill_attvalue(&psAttributeElement->sAttributeValue);

}



int fill_attvalue(DbvAttributeValue *psAttributeValue)
{
	int choice = 0;
        char choice_string[100];


	
        choice = 0;
        while (choice < 1 || choice > 2) {
                printf("Enter eAttributeValueType type: numeric (1), complex (2): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:
		psAttributeValue->eAttributeValueType = AVT_Numeric;
		psAttributeValue->uAttributeValue.ulNumeric = 987;

		break;
	case 2:

		psAttributeValue->eAttributeValueType = AVT_Complex;
		psAttributeValue->uAttributeValue.psComplexValue = (DbvComplexValue *) DbvAlloc (sizeof(DbvComplexValue));
		fill_complex(psAttributeValue->uAttributeValue.psComplexValue);
		break;
	}
}



int fill_complex(DbvComplexValue *psComplexValue)
{
	/* First compulsory */ 
	/* Create three list nodes, 0, 1, \0 */



	psComplexValue->ppsList = (DbvStringOrNumeric **) DbvAlloc (sizeof(DbvStringOrNumeric *) * 3);
	psComplexValue->ppsList[0] = (DbvStringOrNumeric *) DbvAlloc (sizeof(DbvStringOrNumeric));
	printf("Complex Node 0: ");
	fill_strornum(psComplexValue->ppsList[0], SNT_String);

	psComplexValue->ppsList[1] = (DbvStringOrNumeric *) DbvAlloc (sizeof(DbvStringOrNumeric));
	printf("Complex Node 1: ");
	fill_strornum(psComplexValue->ppsList[1], SNT_Numeric);


	/* Terminate the list */
	psComplexValue->ppsList[2] = NULL;
	



	/* Second optional */

	/* Create three list nodes, 0, 1, \0 */

	/* Create the list */


	psComplexValue->ppulSemanticAction = (Ulong **) DbvAlloc (sizeof(Ulong *) * 3);

	/* Create and fill in a list node */
	psComplexValue->ppulSemanticAction[0] = (Ulong *) DbvAlloc (sizeof(Ulong));
	*psComplexValue->ppulSemanticAction[0] = 111;


	/* Create and fill in next list node */
	psComplexValue->ppulSemanticAction[1] = (Ulong *) DbvAlloc (sizeof(Ulong));
	*psComplexValue->ppulSemanticAction[1] = 222;


	/* Terminate the list */
	psComplexValue->ppulSemanticAction[2] = NULL;


}



void fill_DbvTerm(DbvTerm *psTerm, DbvTermType eTermType)
{
	fill_term(psTerm, eTermType);
}



int fill_term(DbvTerm *psTerm, DbvTermType eTermType)
{
	char *test_general = "this is a general term test string";
	unsigned long test_integer = 55;
	char *test_simpchar = "this is a simple character term test string";

	char *test_oid = ANSI_UKMARC;

	/* Generalized time - see the ASN.1 (not Z39.50) spec 
           1995 (1995), Feb (02), 16th (16), nine oclock (21), six minutes (06),
      	   27.3 seconds (27.3) */
	char *test_datetime = "19950216210627.3";

	char *test_sutrs_external = "this is a term test sutrs external"; 


	switch (eTermType) {
	case TT_General:
		psTerm->eTermType = TT_General;
		psTerm->uTerm.psGeneral = (DbvOctetString *) DbvAlloc (sizeof(DbvOctetString));
		fill_dbv_octet_string(psTerm->uTerm.psGeneral, test_general);

		break;
	case TT_Numeric:
		psTerm->eTermType = TT_Numeric;
		psTerm->uTerm.ulInteger = test_integer;
		break;
	case TT_SimpleCharacter:
		psTerm->eTermType = TT_SimpleCharacter;
		psTerm->uTerm.psSimpleCharacter = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psTerm->uTerm.psSimpleCharacter, test_simpchar);
		break;
	case TT_ObjectIdentifier:
		psTerm->eTermType = TT_ObjectIdentifier;
		psTerm->uTerm.pcObjectIdentifier = (DbvObjectIdentifier ) DbvAlloc (strlen(test_oid) +1);
		memcpy(psTerm->uTerm.pcObjectIdentifier, test_oid, strlen(test_oid));
		break;
	case TT_DateTime:
		psTerm->eTermType = TT_DateTime;
		psTerm->uTerm.pcDateTime = (DbvGeneralizedTime ) DbvAlloc (strlen(test_datetime) +1);
		memcpy(psTerm->uTerm.pcDateTime, test_datetime, strlen(test_datetime));
		break;
	case TT_External:
		psTerm->eTermType = TT_External;
		psTerm->uTerm.psExternal = (DbvASN1External *) DbvAlloc (sizeof(DbvASN1External));
		fill_dbv_sutrs_external(psTerm->uTerm.psExternal, test_sutrs_external);

		break;
	case TT_IntegerAndUnit:
		psTerm->eTermType = TT_IntegerAndUnit;
		psTerm->uTerm.psIntegerAndUnit = (DbvIntUnit *) DbvAlloc (sizeof(DbvIntUnit));
		fill_intunit(psTerm->uTerm.psIntegerAndUnit);

		break;
	case TT_Null:
		psTerm->eTermType = TT_Null;
		break;
	}
}







int fill_records_multidiag(DbvDiagRec **ppsMultipleNonSurrogateDiag)
{
        DbvASN1External *external = NULL;
        DbvASN1External *external0 = NULL;
        DbvASN1External *external1 = NULL;
        DbvASN1External *external2 = NULL;
 
        ppsMultipleNonSurrogateDiag[0] = (DbvDiagRec *) DbvAlloc (sizeof (DbvDiagRec));
 
        /* DbvDefaultDiagFormat V2 */
        fill_default_diag_rec(ppsMultipleNonSurrogateDiag[0], ANSI_LIBRISMARC, 33, "another diag text for v2", PVIF_Version2, DT_DefaultFormat, NULL);
 
        ppsMultipleNonSurrogateDiag[1] = (DbvDiagRec *) DbvAlloc (sizeof (DbvDiagRec));
 
        /* DbvASN1External */
 
        /* Create an external and pass it down */
        external = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
        fill_dbv_sutrs_external(external, "this is a sutrs external being used in a multiple non surrogate diagnostic");
 
        fill_default_diag_rec(ppsMultipleNonSurrogateDiag[1], NULL, 0, NULL, 0, DT_ExternallyDefined, external);
 
 
        ppsMultipleNonSurrogateDiag[2] = (DbvDiagRec *) DbvAlloc (sizeof (DbvDiagRec));
 
        /* DbvDiagRec, DbvDefaultDiagFormat V3 */
        fill_default_diag_rec(ppsMultipleNonSurrogateDiag[2], ANSI_DANMARC, 44, "another diag text for v3", PVIF_Version3, DT_DefaultFormat, NULL);
 
        ppsMultipleNonSurrogateDiag[3] = NULL;
}
 


int fill_specification(DbvSpecification *psGeneric)
{
 
        char *oid_value = ANSI_DANMARC;

 
        /* DbvObjectIdentifier is a typedef char * */
 
        /* +1 to allocate space for the \0 */
        psGeneric->pcSchema = (DbvObjectIdentifier ) DbvAlloc (strlen(oid_value) +1);
 
        /* Have an ANSI_DANMARC for the ObjectIdentifier */
	strcpy(psGeneric->pcSchema, ANSI_DANMARC);
 
	/* optional */
	psGeneric->psElementSpec = (DbvElementSpec *) DbvAlloc (sizeof (DbvElementSpec));
	fill_DbvElementSpec(psGeneric->psElementSpec);
}



int fill_DbvElementSpec(DbvElementSpec *psElementSpec)
{
	int choice = 0;
        char choice_string[100];
        char *test_inter_string = "a specification test";
        char sutrs_string[1000];

	choice = 0;
	while (choice < 1 || choice > 2) {
                printf("Enter eElementSpecType: EST_Primitive (1), EST_External (2): ");
                choice = (atol(gets(choice_string)));
        }


	switch (choice) {
	case 1:

                /* An international string */
                psElementSpec->eElementSpecType = EST_Primitive;
                psElementSpec->uElementSpec.psPrimitive = (DbvInternationalString *)DbvAlloc (sizeof (DbvInternationalString));
                fill_dbv_inter_string(psElementSpec->uElementSpec.psPrimitive, test_inter_string);
		break;
	case 2:
                psElementSpec->eElementSpecType = EST_External;
                psElementSpec->uElementSpec.psExternal = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
 
                /* Set up a sutrs external */
                strcpy(sutrs_string, "an external specification sutrs");
                fill_dbv_sutrs_external(psElementSpec->uElementSpec.psExternal, sutrs_string);
		break;
        }

}


 
 
 
int fill_attelmnts(DbvAttributeElement **ppsAttributeElement)
{
        /* First node */
        ppsAttributeElement[0] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
        printf("Node 0: ");
        fill_attelement(ppsAttributeElement[0]);
 
        /* Second node */
        ppsAttributeElement[1] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
        printf("Node 1: ");
        fill_attelement(ppsAttributeElement[1]);
 
        /* NULL list terminator */
        ppsAttributeElement[2] = NULL;
}



int fill_permissions_list(DbvPermissions **ppsPermissions, int number_of_nodes)
{
	int index = 0;

	for (index = 0; index < (number_of_nodes-1); index ++) {

        	ppsPermissions[index] = (DbvPermissions *) DbvAlloc (sizeof(DbvPermissions));
        	fill_dbv_permissions(ppsPermissions[index]);
	}
 
        /* Terminate the list */
        ppsPermissions[index] = NULL;
}



int fill_dbv_permissions(DbvPermissions *psPermissions)
{
        char *test_userid = "test permissions user id";
 
        fill_dbv_inter_string(&psPermissions->sUserId, test_userid);
 
        /* Have four nodes = 0, 1, 2, \0 */
        psPermissions->ppeAllowableFunctions = (DbvPermAllowableFunctions **) DbvAlloc (sizeof (DbvPermAllowableFunctions *) * 4);
 
        psPermissions->ppeAllowableFunctions[0] = (DbvPermAllowableFunctions *) DbvAlloc (sizeof (DbvPermAllowableFunctions));
        *psPermissions->ppeAllowableFunctions[0] = PAF_Delete;
        psPermissions->ppeAllowableFunctions[1] = (DbvPermAllowableFunctions *) DbvAlloc (sizeof (DbvPermAllowableFunctions));
        *psPermissions->ppeAllowableFunctions[1] = PAF_ModifyContents;

        psPermissions->ppeAllowableFunctions[2] = (DbvPermAllowableFunctions *) DbvAlloc (sizeof (DbvPermAllowableFunctions));
        *psPermissions->ppeAllowableFunctions[2] = PAF_ModifyPermissions;

 
        /* Terminate the list */
        psPermissions->ppeAllowableFunctions[3] = NULL;
}

 



int fill_dbv_resrep1_external(DbvASN1External *external)
{
 
        char *oid = ANSI_RESOURCE_REPORT_1;
 
        external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
        memcpy(external->pcObjectIdentifier, oid, strlen(oid));
 
        external->eValueType = EVT_SingleASN1Type;
        external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
        external->uExternalValue.psSingleASN1Type->eASN1Type = AT_ResourceReport1;
        external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport1 = (struct DbvResourceReport1 *) DbvAlloc (sizeof (struct DbvResourceReport1));
 
        fill_dbv_resrep1_main(external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport1);



}



int fill_dbv_resrep1_main(DbvResourceReport1 *psResourceReport1)
{
	/* Both the following are compulsory */
 
        char *message = "resource report 1 message test";
 
        /* Space for 3 nodes - 0, 1 , \0 */
        psResourceReport1->ppsEstimate = (DbvEstimate1 **) DbvAlloc (sizeof(DbvEstimate1 *) * 3);
 
        psResourceReport1->ppsEstimate[0] = (DbvEstimate1 *) DbvAlloc (sizeof (DbvEstimate1));
        fill_dbv_estimate1(psResourceReport1->ppsEstimate[0]);
 
        psResourceReport1->ppsEstimate[1] = (DbvEstimate1 *) DbvAlloc (sizeof (DbvEstimate1));
        fill_dbv_estimate1(psResourceReport1->ppsEstimate[1]);
 
        /* Terminate the list */
        psResourceReport1->ppsEstimate[2] = NULL;

 
        /* Now fill the sMessage */
        fill_dbv_inter_string(&psResourceReport1->sMessage, message);

}
 

 
int fill_dbv_estimate1(DbvEstimate1 *psEstimate)
{
 
        psEstimate->eEstimateType = ET_FinalOpTimeProcessing;
        psEstimate->ulValue = 123;
 
        /* currency code is optional */
        psEstimate->pulCurrencyCode = (Ulong *) DbvAlloc (sizeof (Ulong));
        *psEstimate->pulCurrencyCode = 321;
 
}



int fill_dbv_resrep2_external(DbvASN1External *external)
{
        char *oid = ISO_RESOURCE_REPORT_2;
 
        external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
        memcpy(external->pcObjectIdentifier, oid, strlen(oid));
 
        external->eValueType = EVT_SingleASN1Type;
        external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
        external->uExternalValue.psSingleASN1Type->eASN1Type = AT_ResourceReport2;
        external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport2 = (struct DbvResourceReport2 *) DbvAlloc (sizeof (struct DbvResourceReport2));
 
        fill_dbv_resrep2_main(external->uExternalValue.psSingleASN1Type->uASN1.psResourceReport2);

}



int fill_dbv_resrep2_main(DbvResourceReport2 *psResourceReport2)
{
 
	/* Both the following are optional */

        char *message = "resource report 2 message test";
 
        /* Space for 3 nodes - 0, 1 , \0 */
        psResourceReport2->ppsEstimate = (DbvEstimate2 **) DbvAlloc (sizeof(DbvEstimate2 *) * 3);
 
        psResourceReport2->ppsEstimate[0] = (DbvEstimate2 *) DbvAlloc (sizeof (DbvEstimate2));
        fill_dbv_estimate2(psResourceReport2->ppsEstimate[0]);
 
        psResourceReport2->ppsEstimate[1] = (DbvEstimate2 *) DbvAlloc (sizeof (DbvEstimate2));
        fill_dbv_estimate2(psResourceReport2->ppsEstimate[1]);
 
        /* Terminate the list */
        psResourceReport2->ppsEstimate[2] = NULL;
 
 
        /* Now fill the psMessage */
	psResourceReport2->psMessage = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
        fill_dbv_inter_string(psResourceReport2->psMessage, message);
 
}
 


int fill_dbv_estimate2(DbvEstimate2 *psEstimate)
{
        fill_strornum(&psEstimate->sType, SNT_String);
        fill_intunit(&psEstimate->sValue);
 
}
 

void fill_DbvRange(DbvRange *psRange)
{
	/* Just some arbitrary values */

	psRange->ulStartingPosition = 43;
        psRange->ulNumberOfRecords = 44;

}



void fill_DbvRecordComposition(DbvRecordComposition *psRecordComposition)
{
	char	*test_el_set_name = "present request element set name";
	DbvDbSpecificComp ** convien_ptr = NULL;
	
	char *rec_syn0 = ISO_UNIMARC;
	char *rec_syn1 = ANSI_UNIMARC;
	char *rec_syn2 = ISO_INTERMARC;
        int choice = 0;
        char choice_string[100];

	

        choice = 0;
        while (choice < 1 || choice > 6) {
                printf("Enter psRecordComposition type: elset_generic (1), elset_dbspecific (2), compspec_generic (3), compspec_specific (4), compspec_recsyn (5) 'bigtest' (6): ");
                choice = (atol(gets(choice_string)));
        }
 

	switch(choice) {
	case 1:
		psRecordComposition->eRecordCompositionType = RCT_ElementSetNames;
		psRecordComposition->uRecordComposition.psElementSetNames  = (DbvElementSetNames  * ) DbvAlloc (sizeof (DbvElementSetNames));
		psRecordComposition->uRecordComposition.psElementSetNames->eElementSetType = EST_Generic;
		psRecordComposition->uRecordComposition.psElementSetNames->uElementSet.psGeneric = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psRecordComposition->uRecordComposition.psElementSetNames->uElementSet.psGeneric, test_el_set_name);
		break;
	case 2:
		psRecordComposition->eRecordCompositionType = RCT_ElementSetNames;
		psRecordComposition->uRecordComposition.psElementSetNames  = (DbvElementSetNames  * ) DbvAlloc (sizeof (DbvElementSetNames));
		psRecordComposition->uRecordComposition.psElementSetNames->eElementSetType = EST_DatabaseSpecific;
	

		/* An array of pointers */
		/* Lets have four list nodes this time (0, 1, 2 \0) */

		psRecordComposition->uRecordComposition.psElementSetNames->uElementSet.ppsElementSetsDbSpecific = (DbvElementSetsDbSpecific **) DbvAlloc (sizeof(DbvElementSetsDbSpecific *) * 4);
	
		fill_DbvElementSetsDbSpecific_list(psRecordComposition->uRecordComposition.psElementSetNames->uElementSet.ppsElementSetsDbSpecific);
	
		break;
	case 3:
		psRecordComposition->eRecordCompositionType = RCT_CompSpec;
		psRecordComposition->uRecordComposition.psCompSpec  = (DbvCompSpec * ) DbvAlloc (sizeof (DbvCompSpec));
		psRecordComposition->uRecordComposition.psCompSpec->fSelectAlternativeSyntax = DBV_TRUE;
		psRecordComposition->uRecordComposition.psCompSpec->psGeneric = (DbvSpecification *) DbvAlloc (sizeof (struct DbvSpecification));
		fill_specification(psRecordComposition->uRecordComposition.psCompSpec->psGeneric);
		break;
	case 4:
		psRecordComposition->eRecordCompositionType = RCT_CompSpec;
		psRecordComposition->uRecordComposition.psCompSpec  = (DbvCompSpec * ) DbvAlloc (sizeof (DbvCompSpec));
		psRecordComposition->uRecordComposition.psCompSpec->fSelectAlternativeSyntax = DBV_FALSE;
	
		/* No psGeneric */

		/* Lets have a a list with two elements (+1 for a NULL pointer at the
	   	end) */
		psRecordComposition->uRecordComposition.psCompSpec->ppsDbSpecific = (DbvDbSpecificComp **) DbvAlloc (sizeof (DbvDbSpecificComp *) * 3);


		/* Set up a convenince pointer to the array of pointers to
	   	DbSpecificComp just created */
		convien_ptr = psRecordComposition->uRecordComposition.psCompSpec->ppsDbSpecific;

		convien_ptr[0] = (DbvDbSpecificComp *) DbvAlloc (sizeof(DbvDbSpecificComp));

		/* Dont have to Alloc sDatabaseName or sSpecification */
		fill_dbv_inter_string(&convien_ptr[0]->sDatabaseName, "first specification database name"); 

		fill_specification(&(convien_ptr[0]->sSpecification));

		convien_ptr[1] = (DbvDbSpecificComp *) DbvAlloc (sizeof(DbvDbSpecificComp));

		fill_dbv_inter_string(&convien_ptr[1]->sDatabaseName, "a second specification database name"); 

		fill_specification(&(convien_ptr[1]->sSpecification));


		convien_ptr[2] = NULL;
		break;
	case 5:
		/* The bizarre infinate length list of object Identifiers.... */

		psRecordComposition->eRecordCompositionType = RCT_CompSpec;
		psRecordComposition->uRecordComposition.psCompSpec  = (DbvCompSpec * ) DbvAlloc (sizeof (DbvCompSpec));
		psRecordComposition->uRecordComposition.psCompSpec->fSelectAlternativeSyntax = DBV_TRUE;
	

		/* No psGeneric or ppsDbSpecific */

		/* The following decalares an array of pointers to char * (because
	   	DbvObjectIdentifier is a typedefed char* ) */
		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax = (DbvObjectIdentifier *) DbvAlloc (sizeof (DbvObjectIdentifier *) * 4);

		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[0] = (DbvObjectIdentifier ) DbvAlloc (strlen(rec_syn0) +1);

		memcpy(psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[0], rec_syn0, strlen(rec_syn0));


	
		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[1] = (DbvObjectIdentifier ) DbvAlloc (strlen(rec_syn1) +1);
		memcpy(psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[1], rec_syn1, strlen(rec_syn1));

		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[2] = (DbvObjectIdentifier ) DbvAlloc (strlen(rec_syn2) +1);

		memcpy(psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[2], rec_syn2, strlen(rec_syn2));
	
		/* Terminate the list */
		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[3] = NULL;
		break;
	case 6:
		/* Compulsory stuff */
		psRecordComposition->eRecordCompositionType = RCT_CompSpec;
		psRecordComposition->uRecordComposition.psCompSpec  = (DbvCompSpec * ) DbvAlloc (sizeof (DbvCompSpec));
		psRecordComposition->uRecordComposition.psCompSpec->fSelectAlternativeSyntax = DBV_TRUE;
	
		/* psGeneric */
		psRecordComposition->uRecordComposition.psCompSpec->psGeneric = (DbvSpecification *) DbvAlloc (sizeof (struct DbvSpecification));
	
		fill_specification(psRecordComposition->uRecordComposition.psCompSpec->psGeneric);
	
	
		/* ppsDbSpecific */
		psRecordComposition->uRecordComposition.psCompSpec->ppsDbSpecific = (DbvDbSpecificComp **) DbvAlloc (sizeof (DbvDbSpecificComp *) * 3);
		convien_ptr = psRecordComposition->uRecordComposition.psCompSpec->ppsDbSpecific;
		convien_ptr[0] = (DbvDbSpecificComp *) DbvAlloc (sizeof(DbvDbSpecificComp));
		fill_dbv_inter_string(&convien_ptr[0]->sDatabaseName, "first specification database name"); 
		fill_specification(&(convien_ptr[0]->sSpecification));
	
	
		convien_ptr[1] = (DbvDbSpecificComp *) DbvAlloc (sizeof(DbvDbSpecificComp));
	
		fill_dbv_inter_string(&convien_ptr[1]->sDatabaseName, "a second specification database name"); 
		fill_specification(&(convien_ptr[1]->sSpecification));
		convien_ptr[2] = NULL;
	
	
		/* ppcRecordSyntax */
		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax = (DbvObjectIdentifier *) DbvAlloc (sizeof (DbvObjectIdentifier *) * 2);
	
		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[0] = (DbvObjectIdentifier ) DbvAlloc (strlen(rec_syn0) +1);
		memcpy(psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[0], rec_syn0, strlen(rec_syn0));
	
		psRecordComposition->uRecordComposition.psCompSpec->ppcRecordSyntax[1] = NULL;
		break;
	}
	
	
}




void fill_DbvElementSetsDbSpecific_list(DbvElementSetsDbSpecific **ppsElementSetsDbSpecific)
{
	char *node0_dbname = "Lamborghini";
	char *node0_elset = "Diablo VT";

	char *node1_dbname = "Ferrari";
	char *node1_elset = "F355 berlinetta";

	char *node2_dbname = "Lotus";
	char *node2_elset = "Esprit Sport 300";

	/* There is space in the list for four list nodes so lets 
	    put them in (0, 1, 2, \0) */

	/* dbname and element set name are BOTH COMPULSORY */

	/*********************************************************/
	/* node 0 - Database name and Element set name */
	ppsElementSetsDbSpecific[0] = (DbvElementSetsDbSpecific *) DbvAlloc (sizeof (DbvElementSetsDbSpecific));

	ppsElementSetsDbSpecific[0]->psDatabaseName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(ppsElementSetsDbSpecific[0]->psDatabaseName, node0_dbname);
	ppsElementSetsDbSpecific[0]->psElementSetName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(ppsElementSetsDbSpecific[0]->psElementSetName, node0_elset);
	/*********************************************************/

	/* node 1  */
	ppsElementSetsDbSpecific[1] = (DbvElementSetsDbSpecific *) DbvAlloc (sizeof (DbvElementSetsDbSpecific));
	ppsElementSetsDbSpecific[1]->psDatabaseName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(ppsElementSetsDbSpecific[1]->psDatabaseName, node1_dbname);
	ppsElementSetsDbSpecific[1]->psElementSetName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(ppsElementSetsDbSpecific[1]->psElementSetName, node1_elset);
	
	/*********************************************************/
	/* node 2  */
	ppsElementSetsDbSpecific[2] = (DbvElementSetsDbSpecific *) DbvAlloc (sizeof (DbvElementSetsDbSpecific));
	ppsElementSetsDbSpecific[2]->psDatabaseName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(ppsElementSetsDbSpecific[2]->psDatabaseName, node2_dbname);
	ppsElementSetsDbSpecific[2]->psElementSetName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_dbv_inter_string(ppsElementSetsDbSpecific[2]->psElementSetName, node2_elset);
}



void fill_DbvInternationalString(DbvLanguageCode *psProposedLangauages, char *pcString)
{
	fill_dbv_inter_string(psProposedLangauages, pcString);
}




void fill_DbvCharSetAndLanguage_external(DbvASN1External *external)
{

	char *oid = ANSI_CHARSET_AND_LANGUAGE_NEGOTIATION;

        /* ALWAYS SUPPLY AN OID WITH SINGLE ASN1 TYPE */


        /* +1 to copy the \0 */
        external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
        memcpy(external->pcObjectIdentifier, oid, strlen(oid));

        external->eValueType = EVT_SingleASN1Type;
        external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
        external->uExternalValue.psSingleASN1Type->eASN1Type = AT_CharSetAndLanguage;
        external->uExternalValue.psSingleASN1Type->uASN1.psCharSetAndLanguage = (struct DbvCharSetAndLanguage *) DbvAlloc (sizeof (struct DbvCharSetAndLanguage));

        fill_DbvCharSetAndLanguage(external->uExternalValue.psSingleASN1Type->uASN1.psCharSetAndLanguage);

}



void fill_DbvCharSetAndLanguage(DbvCharSetAndLanguage *psCharSetAndLanguage)
{
	int choice = 0;
	char choice_string[100];

	choice = 0;
	while (choice < 1 || choice > 2) {
		printf("Enter eCharSetAndLanguageType type: CSALT_Proposal (1), CSALT_Response (2): ");
		choice = (atol(gets(choice_string)));
	}
			
	switch (choice) {
	case 1:
		psCharSetAndLanguage->eCharSetAndLanguageType = CSALT_Proposal;
		psCharSetAndLanguage->uCharSetAndLanguage.psProposal = (DbvOriginProposal *) DbvAlloc (sizeof (DbvOriginProposal));
		fill_DbvOriginProposal(psCharSetAndLanguage->uCharSetAndLanguage.psProposal);
		break;
	case 2: 
		psCharSetAndLanguage->eCharSetAndLanguageType = CSALT_Response;
		psCharSetAndLanguage->uCharSetAndLanguage.psResponse = (DbvTargetResponse *) DbvAlloc (sizeof (DbvTargetResponse));
		fill_DbvTargetResponse(psCharSetAndLanguage->uCharSetAndLanguage.psResponse);
		break;
	}
}



void fill_DbvOriginProposal(DbvOriginProposal *psProposal)
{
	/* o */
	psProposal->ppsProposedCharSets = (DbvProposedCharSet **) DbvAlloc (sizeof (DbvProposedCharSet *) * 4);

	psProposal->ppsProposedCharSets[0] = (DbvProposedCharSet *) DbvAlloc (sizeof (DbvProposedCharSet));

	psProposal->ppsProposedCharSets[1] = (DbvProposedCharSet *) DbvAlloc (sizeof (DbvProposedCharSet));
	psProposal->ppsProposedCharSets[2] = (DbvProposedCharSet *) DbvAlloc (sizeof (DbvProposedCharSet));
	psProposal->ppsProposedCharSets[3] = NULL;

	fill_DbvProposedCharSet(psProposal->ppsProposedCharSets[0], PRCST_Iso2022);

	fill_DbvProposedCharSet(psProposal->ppsProposedCharSets[1], PRCST_Iso10646);
	fill_DbvProposedCharSet(psProposal->ppsProposedCharSets[2], PRCST_Private);
	

	/* o */
	psProposal->ppsProposedLanguages = (DbvLanguageCode **) DbvAlloc (sizeof (DbvLanguageCode *) * 3);
	psProposal->ppsProposedLanguages[0] = (DbvLanguageCode *) DbvAlloc (sizeof (DbvLanguageCode));
	psProposal->ppsProposedLanguages[1] = (DbvLanguageCode *) DbvAlloc (sizeof (DbvLanguageCode));
	psProposal->ppsProposedLanguages[2] = NULL;

	fill_DbvLanguageCode(psProposal->ppsProposedLanguages[0]);
	fill_DbvLanguageCode(psProposal->ppsProposedLanguages[1]);

	
	/* o */
	psProposal->pfRecordsInSelectedCharSets = (DbvBool *) DbvAlloc (sizeof (DbvBool));
	*psProposal->pfRecordsInSelectedCharSets = DBV_TRUE;

}



void fill_DbvProposedCharSet(DbvProposedCharSet *psProposedCharSets, DbvProposedCharSetType eProposedCharSetType)
{
	switch (eProposedCharSetType) {
	case PRCST_Iso2022:
		psProposedCharSets->eProposedCharSetType = PRCST_Iso2022;
		psProposedCharSets->uProposedCharSet.psIso2022 = (DbvIso2022 *) DbvAlloc (sizeof (DbvIso2022));
		fill_DbvIso2022(psProposedCharSets->uProposedCharSet.psIso2022, I2022T_OriginProposal);
		break;
	case PRCST_Iso10646: 
		psProposedCharSets->eProposedCharSetType = PRCST_Iso10646;
		psProposedCharSets->uProposedCharSet.psIso10646 = (DbvIso10646 *) DbvAlloc (sizeof (DbvIso10646));
		fill_DbvIso10646(psProposedCharSets->uProposedCharSet.psIso10646);
		break;
	case PRCST_Private: 
		psProposedCharSets->eProposedCharSetType = PRCST_Private;
		psProposedCharSets->uProposedCharSet.psPrivate = (DbvPrivateCharacterSet *) DbvAlloc (sizeof (DbvPrivateCharacterSet));
		fill_DbvPrivateCharacterSet(psProposedCharSets->uProposedCharSet.psPrivate);
		break;
	}
}



void fill_DbvIso2022(DbvIso2022 *psIso2022, DbvIso2022Type eIso2022Type)
{
	switch (eIso2022Type) {
	case I2022T_OriginProposal:
		psIso2022->eIso2022Type = I2022T_OriginProposal;
		psIso2022->uIso2022Type.psOriginProposal = (DbvIso2022Proposal *) DbvAlloc (sizeof (DbvIso2022Proposal));
		fill_DbvIso2022Proposal(psIso2022->uIso2022Type.psOriginProposal);
		break;
	case I2022T_TargetResponse: 
		psIso2022->eIso2022Type = I2022T_TargetResponse;
		psIso2022->uIso2022Type.psTargetResponse = (DbvIso2022Response *) DbvAlloc (sizeof (DbvIso2022Response));
		fill_DbvIso2022Response(psIso2022->uIso2022Type.psTargetResponse);
		break;
	}
}



void fill_DbvIso2022Proposal(DbvIso2022Proposal *psOriginProposal)
{
	/* o */
	psOriginProposal->psProposedEnvironment = (DbvEnvironment *) DbvAlloc (sizeof (DbvEnvironment));
	fill_DbvEnvironment(psOriginProposal->psProposedEnvironment);

	/* c */
	psOriginProposal->ppulProposedSets = (Ulong **) DbvAlloc (sizeof (Ulong *) * 3);
	psOriginProposal->ppulProposedSets[0] = (Ulong *) DbvAlloc (sizeof (Ulong));
	psOriginProposal->ppulProposedSets[1] = (Ulong *) DbvAlloc (sizeof (Ulong));

	*psOriginProposal->ppulProposedSets[0] = 21;
	*psOriginProposal->ppulProposedSets[1] = 22;


	/* c */
	psOriginProposal->ppsProposedInitialSets = (DbvInitialSet **) DbvAlloc (sizeof (DbvInitialSet *) * 3);
	psOriginProposal->ppsProposedInitialSets[0] = (DbvInitialSet *) DbvAlloc (sizeof (DbvInitialSet));
	psOriginProposal->ppsProposedInitialSets[1] = (DbvInitialSet *) DbvAlloc (sizeof (DbvInitialSet));
	fill_DbvInitialSet(psOriginProposal->ppsProposedInitialSets[0]);	
	fill_DbvInitialSet(psOriginProposal->ppsProposedInitialSets[1]);	

	/* c */
	psOriginProposal->psProposedLeftAndRight = (DbvLeftAndRight *) DbvAlloc (sizeof (DbvLeftAndRight));
	fill_DbvLeftAndRight(psOriginProposal->psProposedLeftAndRight);
	
}



void fill_DbvEnvironment(DbvEnvironment *psProposedEnvironment)
{
	/* c */
	psProposedEnvironment->eEnvironmentType = ENT_EightBit;
}



void fill_DbvInitialSet(DbvInitialSet *psProposedInitialSets)
{
	/* 6 c */
	psProposedInitialSets->ulG0 = 661;
	psProposedInitialSets->ulG1 = 662;
	psProposedInitialSets->ulG2 = 663;
	psProposedInitialSets->ulG3 = 664;
	psProposedInitialSets->ulC0 = 665;
	psProposedInitialSets->ulC1 = 666;
	
}


void fill_DbvLeftAndRight(DbvLeftAndRight *psProposedLeftAndRight)
{
	/* 2 c */
	psProposedLeftAndRight->ulGLeft = 87;
	psProposedLeftAndRight->ulGRight = 88;
}



void fill_DbvIso2022Response(DbvIso2022Response *psTargetResponse)
{
	/* c */
	psTargetResponse->psSelectedEnvironment = (DbvEnvironment *) DbvAlloc (sizeof (DbvEnvironment));
	fill_DbvEnvironment(psTargetResponse->psSelectedEnvironment);

	/* c */
	psTargetResponse->ppulSelectedSets = (Ulong **) DbvAlloc (sizeof (Ulong *) * 3);
	psTargetResponse->ppulSelectedSets[0] = (Ulong *) DbvAlloc (sizeof (Ulong));
	psTargetResponse->ppulSelectedSets[1] = (Ulong *) DbvAlloc (sizeof (Ulong));

	*psTargetResponse->ppulSelectedSets[0] = 41;
	*psTargetResponse->ppulSelectedSets[1] = 42;


	/* c */
	psTargetResponse->psSelectedInitialSet = (DbvInitialSet *) DbvAlloc (sizeof (DbvInitialSet));
	fill_DbvInitialSet(psTargetResponse->psSelectedInitialSet);	

	/* c */
	psTargetResponse->psSelectedLeftAndRight = (DbvLeftAndRight *) DbvAlloc (sizeof (DbvLeftAndRight));
	fill_DbvLeftAndRight(psTargetResponse->psSelectedLeftAndRight);
}



void fill_DbvIso10646(DbvIso10646 *psIso10646)
{
	char *oid1 = ISO_UNIMARC, *oid2 = ANSI_SBN;

	/* c */
	psIso10646->pcCollections = (DbvObjectIdentifier) DbvAlloc (strlen(oid1) + 1);
	memcpy(psIso10646->pcCollections, oid1, strlen(oid1));

	/* c */
	psIso10646->pcEncodingLevel = (DbvObjectIdentifier) DbvAlloc (strlen(oid2) + 1);
	memcpy(psIso10646->pcEncodingLevel, oid2, strlen(oid2));
}



void fill_DbvPrivateCharacterSet(DbvPrivateCharacterSet *psPrivate)
{
	char *oid1 = ISO_CCF, *oid2 = ANSI_CCF;

	int choice = 0;
	char choice_string[100];

	choice = 0;
	while (choice < 1 || choice > 3) {
		printf("Enter ePrivateCharacterSetType: PCST_ViaOid (1), PCST_ExternallySpecified (2), PCST_PreviouslyAgreedUpon (3): ");
		choice = (atol(gets(choice_string)));
	}
			
	switch (choice) {
	case 1:
		psPrivate->ePrivateCharacterSetType = PCST_ViaOid;
		psPrivate->uPrivateCharacterSet.ppcViaOid = (DbvObjectIdentifier *) DbvAlloc (sizeof (DbvObjectIdentifier *) * 3);
		psPrivate->uPrivateCharacterSet.ppcViaOid[0] = (DbvObjectIdentifier) DbvAlloc (strlen(oid1) + 1);
		psPrivate->uPrivateCharacterSet.ppcViaOid[1] = (DbvObjectIdentifier) DbvAlloc (strlen(oid2) + 1);

		memcpy(psPrivate->uPrivateCharacterSet.ppcViaOid[0], oid1, strlen(oid1));
		memcpy(psPrivate->uPrivateCharacterSet.ppcViaOid[1], oid2, strlen(oid2));

		break;
	case 2: 
		psPrivate->ePrivateCharacterSetType = PCST_ExternallySpecified;
		psPrivate->uPrivateCharacterSet.psExternallySpecified = (DbvASN1External *) DbvAlloc (sizeof (DbvASN1External));
		fill_dbv_sutrs_external(psPrivate->uPrivateCharacterSet.psExternallySpecified, "sutrs test string");
		break;
	case 3:
		psPrivate->ePrivateCharacterSetType = PCST_PreviouslyAgreedUpon;
		/* Nothing lse to set up */
		break;
	}
}



void fill_DbvLanguageCode(DbvLanguageCode *psProposedLangauages)
{
	fill_DbvInternationalString((DbvInternationalString *) psProposedLangauages, "test language code");
	
}



void fill_DbvTargetResponse(DbvTargetResponse *psResponse)
{
	/* o */
	psResponse->psSelectedCharSets = (DbvSelectedCharSet *) DbvAlloc (sizeof (DbvSelectedCharSet));
	fill_DbvSelectedCharSet(psResponse->psSelectedCharSets);
	
	/* o */
	psResponse->psSelectedLanguage = (DbvGeneralString *) DbvAlloc (sizeof (DbvGeneralString));
	fill_DbvGeneralString(psResponse->psSelectedLanguage, "test general string");

	/* o */
	psResponse->pfRecordsInSelectedCharSets = (DbvBool *) DbvAlloc (sizeof (DbvBool));
	*psResponse->pfRecordsInSelectedCharSets = DBV_FALSE;
}



void fill_DbvSelectedCharSet(DbvSelectedCharSet *psSelectedCharSets)
{
	int choice = 0;
	char choice_string[100];

	choice = 0;
	while (choice < 1 || choice > 4) {
		printf("Enter eSelectedCharSetType: SCST_Iso2022 (1), SCST_Iso10646 (2), SCST_Private (3), SCST_None (4): ");
		choice = (atol(gets(choice_string)));
	}
			
	switch (choice) {
	case 1:
		psSelectedCharSets->eSelectedCharSetType = SCST_Iso2022;
		psSelectedCharSets->uSelectedCharSet.psIso2022 = (DbvIso2022 *) DbvAlloc (sizeof (DbvIso2022));
		fill_DbvIso2022(psSelectedCharSets->uSelectedCharSet.psIso2022, I2022T_TargetResponse);
		break;
	case 2: 
		psSelectedCharSets->eSelectedCharSetType = SCST_Iso10646;
		psSelectedCharSets->uSelectedCharSet.psIso10646 = (DbvIso10646 *) DbvAlloc (sizeof (DbvIso10646));
		fill_DbvIso10646(psSelectedCharSets->uSelectedCharSet.psIso10646);
		break;
	case 3: 
		psSelectedCharSets->eSelectedCharSetType = SCST_Private;
		psSelectedCharSets->uSelectedCharSet.psPrivate = (DbvPrivateCharacterSet *) DbvAlloc (sizeof (DbvPrivateCharacterSet));
		fill_DbvPrivateCharacterSet(psSelectedCharSets->uSelectedCharSet.psPrivate);
		break;
	case 4: 
		psSelectedCharSets->eSelectedCharSetType = SCST_None;
		break;
	}
}



void fill_DbvQuery(DbvQuery *psQuery)
{
	char *test_psPrivateQuery="Query - Private Query";

	/* Use Bib-1 for queries */
	char *rpnoid = ANSI_ATTRIBUTE_SET_BIB_1;
	DbvRPNStructure *rpnstruct1 = NULL;
	char hql_query[100];

	char *test_psIso8777Query="Query - Iso8777Query";
	char *test_psZ3958Query="Query - Z3958Query";
	char *test_psNaturalLanguageQuery="Query - NaturalLanguageQuery";


        int choice = 0;
        char choice_string[100];

	while (choice < 1 || choice > 3) {
		printf("Enter attribute set type: ANSI_ATTRIBUTE_SET_BIB_1 (1), 1.2.840.10003.3.3 (2), 1.2.840.1003.3.8 (3): ");
		choice = (atol(gets(choice_string)));
	}


	if (choice == 1)
		rpnoid = ANSI_ATTRIBUTE_SET_BIB_1;
	else if (choice == 2)

		/* Extended services database oid */
		rpnoid = "1.2.840.10003.3.3";
	else
		/* Cimi attribute set */
		rpnoid = "1.2.840.10003.3.8";



        choice = 0;
        while (choice < 1 || choice > 6) {
                printf("Enter psQuery type: private (1), rpn (2), iso8777 (3), z3958 (4), erpn (5), naturallanguage (6): ");
                choice = (atol(gets(choice_string)));
        }


	switch (choice) {
	case 1:

		psQuery->eQueryType = QT_Private; 
		psQuery->uQuery.psPrivateQuery = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psPrivateQuery, test_psPrivateQuery);
		break;
	case 2:

		psQuery->eQueryType = QT_Rpn; 
		psQuery->uQuery.psRpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
		psQuery->uQuery.psRpnQuery->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(rpnoid) + 1);
		memcpy(psQuery->uQuery.psRpnQuery->pcAttributeSetId, rpnoid, strlen(rpnoid));


		print_pre_query_blurb();

		/* Loop until a valid HQL query is entered */
		while (rpnstruct1 == NULL) {
			printf("Enter an HQL query> ");

			/* Get an input line from the user - allow 
			   99 (max - 1) 
		 	   characters in the line at most (0-98) */ 
			if (getline(hql_query, 100)) {
				rpnstruct1 = form_dbv_rpn(hql_query);

				if (rpnstruct1 == NULL)
					printf("Error in HQL Query\n\n");
			}
		}


		/* At this point there is a valid Dbv RPN query in rpnstruct1 */
		/* Assign structures */
		psQuery->uQuery.psRpnQuery->sRpnStructure = *rpnstruct1;

		break;

	case 3:
		psQuery->eQueryType = QT_Iso8777;
		psQuery->uQuery.psIso8777Query = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psIso8777Query, test_psIso8777Query);
		break;
	case 4:
		psQuery->eQueryType = QT_Z3958;
		psQuery->uQuery.psZ3958Query = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psZ3958Query, test_psZ3958Query);
		break;

	case 5:
		psQuery->eQueryType = QT_Erpn;
		psQuery->uQuery.psErpnQuery = (DbvRPNQuery *) DbvAlloc (sizeof(DbvRPNQuery));
		psQuery->uQuery.psErpnQuery->pcAttributeSetId = (DbvObjectIdentifier) DbvAlloc (strlen(rpnoid) + 1);
		memcpy(psQuery->uQuery.psErpnQuery->pcAttributeSetId, rpnoid, strlen(rpnoid));


		/* Rather than use HQL fill a DbvRPNStructure structure 'by hand' */
		fill_rpnstructure(&psQuery->uQuery.psErpnQuery->sRpnStructure);
		break;
	

	case 6:
		psQuery->eQueryType = QT_NaturalLanguage;
		psQuery->uQuery.psNaturalLanguageQuery = (DbvOctetString *)DbvAlloc(sizeof(DbvOctetString));
		fill_dbv_octet_string(psQuery->uQuery.psNaturalLanguageQuery, test_psNaturalLanguageQuery);
		break;
	}

}



int print_pre_query_blurb(void)
{
	printf("\n");
	printf("Please enter an HQL (Hough Query Language) query.\n");
	printf("\n");
	printf("Some HQL query examples:\n");
	printf("\n");
	printf("author: Charles Dickens\n");
	printf("title: Little Dorrit\n");
	printf("subject: Lamborghini and ( Diablo SE Jota or Countach 5000 QV )\n");
	printf("subject: /and gates/ and boolean logic\n");
	printf("title,author: Macbeth and William Shakespeare\n");
	printf("\n");


}



int fill_rpnstructure(DbvRPNStructure *psRpnStructure)
{
	/* Large V2 style RPN structures (without prox) were/can be tested using
	HQL (see fill_DbvQuery(DbvQuery *psQuery) above) */
	 
	int choice = 0;
        char choice_string[100];

	char *test_ressetid = "test resultsetid operand";
	
        choice = 0;
        while (choice < 1 || choice > 2) {
                printf("Enter query type: operand operator operand (1), single operand (2): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:
		psRpnStructure->eRpnStructureType = RST_RpnLopRopOperator;
		psRpnStructure->uRPNStructure.psRpnLopRopOperator = (DbvRPNLopRopOperator *) DbvAlloc (sizeof(DbvRPNLopRopOperator));
		fill_lopropop(psRpnStructure->uRPNStructure.psRpnLopRopOperator);
		break;
	case 2:

		psRpnStructure->eRpnStructureType = RST_RpnOperand;
		psRpnStructure->uRPNStructure.psRpnOperand = (DbvOperand *) DbvAlloc (sizeof(DbvOperand));
		fill_operand(psRpnStructure->uRPNStructure.psRpnOperand);
		break;
	}
		
}



int fill_lopropop(DbvRPNLopRopOperator *psRpnLopRopOperator)
{

	/* All fields compulsory */

	psRpnLopRopOperator->psLeftOperand = (struct DbvRPNStructure *) DbvAlloc (sizeof(struct DbvRPNStructure));
	fill_rpnstructure(psRpnLopRopOperator->psLeftOperand);

	psRpnLopRopOperator->psRightOperand = (struct DbvRPNStructure *) DbvAlloc (sizeof(struct DbvRPNStructure));
	fill_rpnstructure(psRpnLopRopOperator->psRightOperand);

	/* Note that sOperator is a structure */
	fill_operator(&psRpnLopRopOperator->sOperator);

}



int fill_operand(DbvOperand *psRpnOperand)
{
	int choice = 0;
        char choice_string[100];

	char *test_ressetid = "test resultsetid operand";
	
        choice = 0;
        while (choice < 1 || choice > 3) {
                printf("Enter eOperandType type: attplusterm (1), ressetid (2), ressetplusatt (3): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:

		psRpnOperand->eOperandType = OT_AttributePlusTerm; 
		psRpnOperand->uOperand.psAttributesPlusTerm = (DbvAttributesPlusTerm *) DbvAlloc (sizeof(DbvAttributesPlusTerm));
		fill_attsplusterm(psRpnOperand->uOperand.psAttributesPlusTerm);

		break;
	case 2:

		psRpnOperand->eOperandType = OT_ResultSetId; 
		psRpnOperand->uOperand.psResultSetId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
		fill_dbv_inter_string(psRpnOperand->uOperand.psResultSetId, test_ressetid);  
		break;
	case 3:
		psRpnOperand->eOperandType = OT_ResultSetPlusAttributes; 
		psRpnOperand->uOperand.psResultSetPlusAttributes = (DbvResultSetPlusAttributes *) DbvAlloc (sizeof(DbvResultSetPlusAttributes));
		fill_resplusatts(psRpnOperand->uOperand.psResultSetPlusAttributes);

		break;
	}
}



int fill_resplusatts(DbvResultSetPlusAttributes *psResultSetPlusAttributes)
{
	char *test_ressetid = "result set plus attributes resultset id";

 

	/* Both compulsory */
	fill_dbv_inter_string(&psResultSetPlusAttributes->sResultSetId, test_ressetid);


	printf("Attributes:\n");

	/* Create a ppsAttributeElement list with three nodes - 0, 1, \0 */
	psResultSetPlusAttributes->ppsAttributeElement = (DbvAttributeElement **) DbvAlloc (sizeof (DbvAttributeElement *) * 3);

	/* Create and fill in node 0 */	

	psResultSetPlusAttributes->ppsAttributeElement[0] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
	printf("Node 0: ");
	fill_attelement(psResultSetPlusAttributes->ppsAttributeElement[0]);


	/* Create and fill in node 1 */
	psResultSetPlusAttributes->ppsAttributeElement[1] = (DbvAttributeElement *) DbvAlloc (sizeof(DbvAttributeElement));
	printf("Node 1: ");
	fill_attelement(psResultSetPlusAttributes->ppsAttributeElement[1]);


	/* Terminate the list */
	psResultSetPlusAttributes->ppsAttributeElement[2] = NULL;
	
}



int fill_operator(DbvOperator *psOperator)
{
	int choice = 0;
        char choice_string[100];


	
        choice = 0;
        while (choice < 1 || choice > 4) {
                printf("Enter eOperatorType type: or (1), and (2), andnot (3), prox (4): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:
		psOperator->eOperatorType = OT_Or;
		break;
	case 2:
		psOperator->eOperatorType = OT_And;
		break;
	case 3:
		psOperator->eOperatorType = OT_AndNot;
		break;
	case 4:

		psOperator->eOperatorType = OT_Proximity;
		psOperator->psProximityOperator = (DbvProximityOperator *) DbvAlloc (sizeof(DbvProximityOperator));
		fill_proximity(psOperator->psProximityOperator);
		break;
	}
}



int fill_proximity(DbvProximityOperator *psProximityOperator)
{
	int choice = 0;
        char choice_string[100];


	
	/* optional */


	psProximityOperator->pfExclusion = (DbvBool *) DbvAlloc (sizeof(DbvBool));
	*psProximityOperator->pfExclusion = DBV_TRUE;

	/* All the rest are compulsory */
	psProximityOperator->ulDistance = 888;
	psProximityOperator->fOrdered = DBV_FALSE;

        choice = 0;
        while (choice < 1 || choice > 6) {
                printf("Enter eProximityRelationType type: < (1), <= (2), = (3), >= (4), > (5), != (6): ");
                choice = (atol(gets(choice_string)));
        }
	
	switch (choice) {
	case 1:
		psProximityOperator->eProximityRelationType = PRT_LessThan;
		break;
	case 2:
		psProximityOperator->eProximityRelationType = PRT_LessThanOrEqual;
		break;
	case 3:
		psProximityOperator->eProximityRelationType = PRT_Equal;
		break;
	case 4:
		psProximityOperator->eProximityRelationType = PRT_GreaterThanOrEqual;
		break;
	case 5:
		psProximityOperator->eProximityRelationType = PRT_GreaterThan;
		break;
	case 6:
		psProximityOperator->eProximityRelationType = PRT_NotEqual;
		break;
	}
		

	fill_proxunitcode(&psProximityOperator->sProximityUnitCode);
}



int fill_proxunitcode(DbvProximityUnitCode *psProximityUnitCode)
{
	int choice = 0;
        char choice_string[100];

        choice = 0;
        while (choice < 1 || choice > 6) {
                printf("Enter eProximityUnitType type: known (1), private (2): ");
                choice = (atol(gets(choice_string)));
        }

	switch (choice) {
	case 1:
		psProximityUnitCode->eProximityUnitType = PUT_Known;
        	choice = 0;
        	while (choice < 1 || choice > 11) {
               		printf("Enter eKnownProximityUnit type: character (1), word (2), sentence (3), para (4), sect (5), chapter (6), doc (7), elem (8), subelem (9), eltype (10), byte (11): ");
                	choice = (atol(gets(choice_string)));
        	}
	
		switch (choice) {
		case 1:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Character;
			break;
		case 2:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Word;
			break;
		case 3:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Sentence;
			break;
		case 4:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Paragraph;
			break;
		case 5:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Section;
			break;
		case 6:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Chapter;
			break;
		case 7:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Document;
			break;
		case 8:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Element;
			break;
		case 9:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_SubElement;
			break;
		case 10:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_ElementType;
			break;
		case 11:
			psProximityUnitCode->uProximityUnit.eKnownProximityUnit = KPU_Byte;
			break;
		}

		break;
	case 2:
		psProximityUnitCode->eProximityUnitType = PUT_Private;
		psProximityUnitCode->uProximityUnit.ulPrivate = 112233;
		break;
	}
}



void fill_DbvCompSpec(DbvCompSpec *psCompSpec)
{
	char *rec_syn0 = ANSI_UNIMARC;
	DbvDbSpecificComp **convien_ptr = NULL;
	char *pcDatabase0 = "First database";
	char *pcDatabase1 = "Second database";

	/* c */
	psCompSpec->fSelectAlternativeSyntax = DBV_TRUE;

	/* o */
	psCompSpec->psGeneric = (DbvSpecification *) DbvAlloc (sizeof (struct DbvSpecification));
	fill_specification(psCompSpec->psGeneric);

 	/* o */
	psCompSpec->ppsDbSpecific = (DbvDbSpecificComp **) DbvAlloc (sizeof (DbvDbSpecificComp *) * 3);
	convien_ptr = psCompSpec->ppsDbSpecific;
	convien_ptr[0] = (DbvDbSpecificComp *) DbvAlloc (sizeof(DbvDbSpecificComp));
	fill_dbv_inter_string(&convien_ptr[0]->sDatabaseName, pcDatabase0);
	fill_specification(&(convien_ptr[0]->sSpecification));

	convien_ptr[1] = (DbvDbSpecificComp *) DbvAlloc (sizeof(DbvDbSpecificComp));
	fill_dbv_inter_string(&convien_ptr[1]->sDatabaseName, pcDatabase1);
	fill_specification(&(convien_ptr[1]->sSpecification));
	convien_ptr[2] = NULL;


	/* o */ 
	psCompSpec->ppcRecordSyntax = (DbvObjectIdentifier *) DbvAlloc (sizeof (DbvObjectIdentifier *) * 2);
	psCompSpec->ppcRecordSyntax[0] = (DbvObjectIdentifier ) DbvAlloc (strlen(rec_syn0) +1);
	memcpy(psCompSpec->ppcRecordSyntax[0], rec_syn0, strlen(rec_syn0));
	psCompSpec->ppcRecordSyntax[1] = NULL;
 
}






void fill_DbvUserInfoSearchResult1(DbvASN1External *external)
{
	char *oid = ANSI_USERINFO_SEARCHRESULT1;

	external->pcObjectIdentifier = (char *) DbvAlloc (strlen(oid) + 1);
	memcpy(external->pcObjectIdentifier, oid, strlen(oid));

	external->eValueType = EVT_SingleASN1Type;
	external->uExternalValue.psSingleASN1Type = (DbvSingleASN1Type * ) DbvAlloc (sizeof (DbvSingleASN1Type));
	external->uExternalValue.psSingleASN1Type->eASN1Type = AT_UserInfoSearchResult1;
	external->uExternalValue.psSingleASN1Type->uASN1.psUserInfoSearchResult1 = (struct DbvUserInfoSearchResult1 *) DbvAlloc (sizeof (struct DbvUserInfoSearchResult1));
    
	fill_DbvUserInfoSearchResult1_main(external->uExternalValue.psSingleASN1Type->uASN1.psUserInfoSearchResult1);


}



void fill_DbvUserInfoSearchResult1_main(DbvUserInfoSearchResult1 *psOriginSpecificParameter)
{
	/* c */
	psOriginSpecificParameter->ppsSearchInfoReport = (DbvSearchInfoReport **) DbvAlloc (sizeof (DbvSearchInfoReport *) * 3);
	psOriginSpecificParameter->ppsSearchInfoReport[0] = (DbvSearchInfoReport *) DbvAlloc (sizeof (DbvSearchInfoReport));
	psOriginSpecificParameter->ppsSearchInfoReport[1] = (DbvSearchInfoReport *) DbvAlloc (sizeof (DbvSearchInfoReport));
	psOriginSpecificParameter->ppsSearchInfoReport[2] = NULL;

	fill_DbvSearchInfoReport(psOriginSpecificParameter->ppsSearchInfoReport[0]);

	fill_DbvSearchInfoReport(psOriginSpecificParameter->ppsSearchInfoReport[1]);

}



void fill_DbvSearchInfoReport(DbvSearchInfoReport *psSearchInfoReport)
{
	char *pcSubqueryId = "Sub query";

	/* o */
	psSearchInfoReport->psSubqueryId = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_DbvInternationalString(psSearchInfoReport->psSubqueryId, pcSubqueryId);
	/* c */
	psSearchInfoReport->fFullQuery = DBV_TRUE;
	
	/* o */
	psSearchInfoReport->psSubqueryExpression = (DbvQueryExpression *) DbvAlloc (sizeof(DbvQueryExpression));
	fill_DbvQueryExpression(psSearchInfoReport->psSubqueryExpression);

	/* o */
	psSearchInfoReport->psSubqueryInterpretation = (DbvQueryExpression *) DbvAlloc (sizeof(DbvQueryExpression));
	fill_DbvQueryExpression(psSearchInfoReport->psSubqueryInterpretation);

	/* o */
	psSearchInfoReport->psSubqueryRecommendation = (DbvQueryExpression *) DbvAlloc (sizeof(DbvQueryExpression));
	fill_DbvQueryExpression(psSearchInfoReport->psSubqueryRecommendation);

	/* o */
	psSearchInfoReport->pulSubqueryCount = (Ulong *) DbvAlloc (sizeof(Ulong));
	*psSearchInfoReport->pulSubqueryCount = 982;


	/* o */
	psSearchInfoReport->psSubqueryWeight = (DbvIntUnit *) DbvAlloc (sizeof(DbvIntUnit));
	fill_DbvIntUnit(psSearchInfoReport->psSubqueryWeight);


	/* o */
	psSearchInfoReport->ppsResultsByDB = (DbvResultsByDB **) DbvAlloc (sizeof(DbvResultsByDB *) * 3);
	psSearchInfoReport->ppsResultsByDB[0] = (DbvResultsByDB *) DbvAlloc (sizeof(DbvResultsByDB));
	psSearchInfoReport->ppsResultsByDB[1] = (DbvResultsByDB *) DbvAlloc (sizeof(DbvResultsByDB));
	psSearchInfoReport->ppsResultsByDB[2] = NULL;
	fill_DbvResultsByDB(psSearchInfoReport->ppsResultsByDB[0]);
	fill_DbvResultsByDB(psSearchInfoReport->ppsResultsByDB[1]);
}



void fill_DbvQueryExpression(DbvQueryExpression *psSubqueryExpression)
{
	int choice = 0;
	char choice_string[100];

	choice = 0;
	while (choice < 1 || choice > 2) {
		printf("Enter eQueryExpressionType type: QET_Term (1), QET_Query (2): ");
		choice = (atol(gets(choice_string)));
	}

	switch (choice) {
	case 1:
		psSubqueryExpression->eQueryExpressionType = QET_Term;
		psSubqueryExpression->uQueryExpression.psTerm = (DbvQueryExpressionSeq *) DbvAlloc (sizeof (DbvQueryExpressionSeq));
		fill_DbvQueryExpressionSeq(psSubqueryExpression->uQueryExpression.psTerm);
		break;
	case 2:
		psSubqueryExpression->eQueryExpressionType = QET_Query;
		psSubqueryExpression->uQueryExpression.psQuery = (DbvQuery *) DbvAlloc (sizeof (DbvQuery));
		fill_DbvQuery(psSubqueryExpression->uQueryExpression.psQuery);
		break;
	}
}



void fill_DbvQueryExpressionSeq(DbvQueryExpressionSeq *psTerm)
{
	char *pcTermComment = "Term comment";

	/* c */
	fill_DbvTerm(&psTerm->sQueryTerm, TT_IntegerAndUnit);

	/* o */
	psTerm->psTermComment = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_DbvInternationalString(psTerm->psTermComment, pcTermComment);
	
}



void fill_DbvResultsByDB(DbvResultsByDB *psresultsByDB)
{
	char *pcResultSetName = "Result Name";

	/* c */
	fill_DbvResultsByDBChoice(&psresultsByDB->sDatabases);

	/* o */
	psresultsByDB->pulCount = (Ulong *) DbvAlloc (sizeof(Ulong));
	*psresultsByDB->pulCount = 98;

	/* o */
	psresultsByDB->psResultSetName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));
	fill_DbvInternationalString(psresultsByDB->psResultSetName, pcResultSetName);

	/* A wee passing note:

	   A malloc without setting such as just

	   psresultsByDB->psResultSetName = (DbvInternationalString *) DbvAlloc (sizeof(DbvInternationalString));

 	   will be ignored when a PDU is encoded..the content will be printed as
	   '' at the sending end but NULL at the receiving end 
    	   (see DBVdbv_to_snacc_inter_string in z3950_shared.c) 

	   When a list is used however, each element of the list will be
	   allocated and encoded irrespsective of the element setting..
           the content will be printed as NULL at both the sending and
	   receiving end when an element is not set
	*/



}



void fill_DbvResultsByDBChoice(DbvResultsByDBChoice *psDatabases)
{
	int choice = 0;
	char choice_string[100];

	choice = 0;
	while (choice < 1 || choice > 2) {
		printf("Enter eResultsByDBChoiceType type: RBDSCT_All (1), RBDSCT_List (2): ");
		choice = (atol(gets(choice_string)));
	}

	switch (choice) {
	case 1:
		psDatabases->eResultsByDBChoiceType = RBDSCT_All;
		break;
	case 2:
		psDatabases->eResultsByDBChoiceType = RBDSCT_List;
		psDatabases->uResultsByDBChoice.ppsList = (DbvInternationalString **) DbvAlloc (sizeof (DbvInternationalString *) * 3);
		psDatabases->uResultsByDBChoice.ppsList[0] = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		psDatabases->uResultsByDBChoice.ppsList[1] = (DbvInternationalString *) DbvAlloc (sizeof (DbvInternationalString));
		psDatabases->uResultsByDBChoice.ppsList[2] = NULL;
		fill_DbvInternationalString(psDatabases->uResultsByDBChoice.ppsList[0], "list element 0");
		fill_DbvInternationalString(psDatabases->uResultsByDBChoice.ppsList[1], "list element 1");
		break;
	}
}




