/*  Module Name: @(#)origin_z3950_search.c	7.2
    Version: 11/28/97
    Author: John Hough/Chris Attewell
    
    Crossnet Systems Ltd.
    Unit 3, The Paddock,
    Hambridge Road,
    Newbury,
    Berkshire, RG14 5TQ

    Date of Issue:  

    QA Acceptance:  
    Date of Acceptance:  

    Description: Origin end routines to create and send a SearchRequest PDU 
		 to a target and process a received SearchResponse PDU 
		 sent by a target

*/

#include "origin_z3950_search.h"

/*****************************************************************
******************************************************************
******************************************************************
CREATE SearchRequest PDU TO SEND TO TARGET AND SEND IT
*/



int DbvSearchRequest(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{
	PDU pdu_to_encode;
	PDU *PDU_ptr;
	DbvProtocolStack eProtocolStack;
	int return_value = SI_NOTOK;
	int val;
	jmp_buf env;

	return_value = SI_NOTOK;
	DBVEnterFuncTrace("DBVSearchRequest",lAssocId);

	if (DBVCheckDbvSearchReqFields(lAssocId, psSearchReq, ppsSIError) == SI_OK)
		if (DBVChangeState(lAssocId, DBV_OE_SEARCHREQ, ppsSIError) == SI_OK)
			if (SIWhichStack(lAssocId, &eProtocolStack, ppsSIError) == SI_OK)
				/* PDU can be created and sent */
				return_value = SI_OK;

	if(return_value == SI_OK)
	{
		DBVSendingTrace(lAssocId);
                DBVSrchReqTrace(lAssocId, psSearchReq);

		PDU_ptr = &pdu_to_encode;

		/* Do a setjump - the following structure setup routines do
                   a longjump to here if allocing fails */
                if ((val = setjmp(env)) == 0) {
 
			PDU_ptr->a.searchRequest = (struct SearchRequest *) Asn1Alloc(sizeof(struct SearchRequest));
                        CheckAsn1Alloc(PDU_ptr->a.searchRequest, env);
			PDU_ptr->choiceId = PDU_SEARCHREQUEST;
 
                        DBVsetup_snacc_srchrequest(PDU_ptr, psSearchReq, env);
                } else {
                        DBVfill_error_structure(lAssocId, SI_Interface_Error, 0, DBV_PDU_OUTOFMEM, DBV_PDU_OUTOFMEM_S, 0, Association_Open, ppsSIError);
                        return SI_NOTOK;
                }

		return_value = DBVencode_and_send(PDU_ptr, lAssocId, ppsSIError);
	}

	DBVLeaveFuncTrace("DBVSearchRequest",lAssocId, return_value);
	return return_value;
}



int DBVCheckDbvSearchReqFields(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{

        if (DBVcheck_dbv_srchreq_replace(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_dbnames(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_sselnam(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_mselnam(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_prefsyn(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_query(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_addinf(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

        if (DBVcheck_dbv_srchreq_othinfo(lAssocId, psSearchReq, ppsSIError) == SI_NOTOK)
                return SI_NOTOK;

	return SI_OK;

}



int DBVcheck_dbv_srchreq_replace(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

        if (DBVcheck_dbv_bool(psSearchReq->fReplaceIndicator) == SI_NOTOK) 
		return(DBVdbv_bool_error(lAssocId, "psSearchReq->fReplaceIndicator", ppsSIError));
 
        return SI_OK;
}



int DBVcheck_dbv_srchreq_dbnames(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

	/* If no list at all */
	if (psSearchReq->ppsDatabaseNames == NULL) 
		return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->ppsDatabaseNames", ppsSIError));
	
	
	return SI_OK;
}



int DBVcheck_dbv_srchreq_sselnam(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{
	/* If null then ok because optional */
	if (psSearchReq->psSmallSetElementNames == NULL)
		return SI_OK;

	switch (psSearchReq->psSmallSetElementNames->eElementSetType) {
	case EST_Generic:

		if (psSearchReq->psSmallSetElementNames->uElementSet.psGeneric == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psSmallSetElementNames->uElementSet.psGeneric", ppsSIError));

		break;
	case EST_DatabaseSpecific:

		if (psSearchReq->psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific == NULL)
			return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific", ppsSIError));


		return(DBVcheck_dbspecific(lAssocId, psSearchReq->psSmallSetElementNames->uElementSet.ppsElementSetsDbSpecific, "psSearchReq->psSmallSetElementNames->uElementSet.", ppsSIError));
		break;
	default:
		return(DBVdbv_invalid_enum_error(lAssocId, "psSearchReq->psSmallSetElementNames->eElementSetType", ppsSIError));
		break;
	}

	return SI_OK;
}





int DBVcheck_dbv_srchreq_mselnam(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

	/* If null then ok because optional */
	if (psSearchReq->psMediumSetElementNames == NULL)
		return SI_OK;

	switch (psSearchReq->psMediumSetElementNames->eElementSetType) {
	case EST_Generic:

              if (psSearchReq->psMediumSetElementNames->uElementSet.psGeneric == NULL)
                        return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psMediumSetElementNames->uElementSet.psGeneric", ppsSIError));

		break;
	case EST_DatabaseSpecific:

              if (psSearchReq->psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific == NULL)
                        return(DBVdbv_field_missing_error(lAssocId, "psSearchReq->psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific", ppsSIError));

		return(DBVcheck_dbspecific(lAssocId, psSearchReq->psMediumSetElementNames->uElementSet.ppsElementSetsDbSpecific, "psSearchReq->psMediumSetElementNames->uElementSet.", ppsSIError));
		break;
	default:
		return(DBVdbv_invalid_enum_error(lAssocId, "psSearchReq->psMediumSetElementNames->eElementSetType", ppsSIError));
		break;
	}

	return SI_OK;
}



int DBVcheck_dbv_srchreq_prefsyn(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{
	/* Can be null because optional */
	if (psSearchReq->pcPreferredRecordSyntax == NULL) 
		return SI_OK;


	if (DBVcheck_dbv_recsyn(psSearchReq->pcPreferredRecordSyntax) == SI_NOTOK) 
		return(DBVdbv_oid_error(lAssocId, "psSearchReq->pcPreferredRecordSyntax", ppsSIError));



	return SI_OK;
	
}



int DBVcheck_dbv_srchreq_query(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError) 
{

        char local_error_buffer[SA_ARRAY_SIZE];

        DBVsa_strcpy(local_error_buffer, "psSearchReq->psQuery");

	if(psSearchReq->psQuery == NULL)
		return(DBVdbv_field_missing_error(lAssocId, local_error_buffer, ppsSIError));

	DBVsa_strcat(local_error_buffer, "->");
	return(DBVcheck_DbvQuery(lAssocId, psSearchReq->psQuery, local_error_buffer, ppsSIError));
}



int DBVcheck_dbv_srchreq_addinf(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{
	/* Can be NULL because optional */
        if (psSearchReq->ppsAdditionalSearchInfo == NULL)
                return SI_OK;


	return(DBVcheck_dbv_otherinfo(lAssocId, psSearchReq->ppsAdditionalSearchInfo, "psSearchReq->ppsAdditionalSearchInfo", ppsSIError));
}



int DBVcheck_dbv_srchreq_othinfo(DbvAssocId lAssocId, DbvSearchReq *psSearchReq, DbvError **ppsSIError)
{
	/* Can be NULL because optional */
	if (psSearchReq->ppsOtherInformation == NULL)
                return SI_OK;


	return(DBVcheck_dbv_otherinfo(lAssocId, psSearchReq->ppsOtherInformation, "psSearchReq->ppsOtherInformation", ppsSIError));
}



int DBVsetup_snacc_srchrequest(PDU *PDU_ptr, DbvSearchReq *psSearchReq, jmp_buf env)
{
	DBVset_snacc_searchreq_refid(PDU_ptr, psSearchReq->psReferenceId, env);
	DBVset_snacc_searchreq_ssetupper(PDU_ptr,  psSearchReq->ulSmallSetUpperBound, env);
	DBVset_snacc_searchreq_lsetlower(PDU_ptr, psSearchReq->ulLargeSetLowerBound, env);
	DBVset_snacc_searchreq_msetpres(PDU_ptr, psSearchReq->ulMediumSetPresentNumber, env);
	DBVset_snacc_searchreq_replace(PDU_ptr, psSearchReq->fReplaceIndicator, env);
	DBVset_snacc_searchreq_resultset(PDU_ptr, psSearchReq->sResultSetName, env);
	DBVset_snacc_searchreq_dbnames(PDU_ptr, psSearchReq->ppsDatabaseNames, env);
	DBVset_snacc_searchreq_ssetelname(PDU_ptr, psSearchReq->psSmallSetElementNames, env);
	DBVset_snacc_searchreq_msetelname(PDU_ptr, psSearchReq->psMediumSetElementNames, env);
	DBVset_snacc_searchreq_prefrecsyn(PDU_ptr, psSearchReq->pcPreferredRecordSyntax, env);
	DBVset_snacc_searchreq_query(PDU_ptr, psSearchReq->psQuery, env);
	DBVset_snacc_searchreq_addinfo(PDU_ptr, psSearchReq->ppsAdditionalSearchInfo, env);
	DBVset_snacc_searchreq_otherinfo(PDU_ptr, psSearchReq->ppsOtherInformation, env);
}


int DBVset_snacc_searchreq_refid(PDU *PDU_ptr, DbvOctetString *psReferenceId, jmp_buf env)
{

	/* Check if a referenceId is given(it is an optional field) */
	if  (psReferenceId != NULL) {
		DBVdbv_to_snacc_octet_string(&(PDU_ptr->a.searchRequest->referenceId), psReferenceId, env);
	}

}


int DBVset_snacc_searchreq_ssetupper(PDU *PDU_ptr,  Ulong ulSmallSetUpperBound, jmp_buf env)
{
	PDU_ptr->a.searchRequest->smallSetUpperBound = ulSmallSetUpperBound;
}



int DBVset_snacc_searchreq_lsetlower(PDU *PDU_ptr, Ulong ulLargeSetLowerBound, jmp_buf env)
{
	PDU_ptr->a.searchRequest->largeSetLowerBound = ulLargeSetLowerBound;
}



int DBVset_snacc_searchreq_msetpres(PDU *PDU_ptr, Ulong ulMediumSetPresentNumber, jmp_buf env)
{
	PDU_ptr->a.searchRequest->mediumSetPresentNumber = ulMediumSetPresentNumber;
}



int DBVset_snacc_searchreq_replace(PDU *PDU_ptr, DbvBool fReplaceIndicator, jmp_buf env)
{
	if (fReplaceIndicator == DBV_TRUE)
		PDU_ptr->a.searchRequest->replaceIndicator = TRUE;
	else
		PDU_ptr->a.searchRequest->replaceIndicator = FALSE;
}



int DBVset_snacc_searchreq_resultset(PDU *PDU_ptr, DbvInternationalString sResultSetName, jmp_buf env)
{
	/* This field is compulsory */
	DBVdbv_to_snacc_inter_string(&(PDU_ptr->a.searchRequest->resultSetName), &sResultSetName, env);
}



int DBVset_snacc_searchreq_dbnames(PDU *PDU_ptr, DbvInternationalString **ppsDatabaseNames, jmp_buf env)
{
	DatabaseName *data_ptr;
        DatabaseName **data_ptr_address;

	int current_listnode = 0;

	PDU_ptr->a.searchRequest->databaseNames = AsnListNew(sizeof(void *));
	CheckAsn1Alloc(PDU_ptr->a.searchRequest->databaseNames, env);
	
	while (ppsDatabaseNames[current_listnode] != NULL) {
		data_ptr_address = AsnListAppend(PDU_ptr->a.searchRequest->databaseNames);
		CheckAsn1Alloc(data_ptr_address, env);

		*data_ptr_address = Asn1Alloc(sizeof(DatabaseName));
		CheckAsn1Alloc(*data_ptr_address, env);

		data_ptr = *data_ptr_address;

		DBVdbv_to_snacc_inter_string(data_ptr, ppsDatabaseNames[current_listnode], env);
		current_listnode++;
	}
		
}



int DBVset_snacc_searchreq_ssetelname(PDU *PDU_ptr, DbvElementSetNames *psSmallSetElementNames, jmp_buf env)
{
	if(psSmallSetElementNames != NULL) {
		PDU_ptr->a.searchRequest->smallSetElementSetNames = (struct ElementSetNames *) Asn1Alloc (sizeof(struct ElementSetNames));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->smallSetElementSetNames, env);

		DBVdbv_to_snacc_el_set_names(PDU_ptr->a.searchRequest->smallSetElementSetNames, psSmallSetElementNames, env);
	}
}



int DBVset_snacc_searchreq_msetelname(PDU *PDU_ptr, DbvElementSetNames *psMediumSetElementNames, jmp_buf env)
{
	if(psMediumSetElementNames != NULL) {
		PDU_ptr->a.searchRequest->mediumSetElementSetNames = (struct ElementSetNames*) Asn1Alloc (sizeof(struct ElementSetNames));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->mediumSetElementSetNames, env);

		DBVdbv_to_snacc_el_set_names(PDU_ptr->a.searchRequest->mediumSetElementSetNames, psMediumSetElementNames, env);
	}
}



int DBVset_snacc_searchreq_prefrecsyn(PDU *PDU_ptr, DbvObjectIdentifier pcPreferredRecordSyntax, jmp_buf env)
{

	if(pcPreferredRecordSyntax != NULL) 
		ASN_MakeOid((char *)pcPreferredRecordSyntax, &(PDU_ptr->a.searchRequest->preferredRecordSyntax), env);
}



int DBVset_snacc_searchreq_query(PDU *PDU_ptr, DbvQuery *psQuery, jmp_buf env)
{
	PDU_ptr->a.searchRequest->query = (Query*) Asn1Alloc (sizeof(Query));
	CheckAsn1Alloc(PDU_ptr->a.searchRequest->query, env);

	DBVdbv_to_snacc_DbvQuery(PDU_ptr->a.searchRequest->query, psQuery, env);
}



int DBVset_snacc_searchreq_addinfo(PDU *PDU_ptr, DbvOtherInformation **ppsAdditionalSearchInfo, jmp_buf env)
{
	/* Check if ppsOtherInformation exists (it is an optional field) */
	if (ppsAdditionalSearchInfo != NULL)  {
		PDU_ptr->a.searchRequest->additionalSearchInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->additionalSearchInfo, env);
		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.searchRequest->additionalSearchInfo, ppsAdditionalSearchInfo, env);
	}
}



int DBVset_snacc_searchreq_otherinfo(PDU *PDU_ptr, DbvOtherInformation **ppsOtherInformation, jmp_buf env)
{
	/* Check if psOtherInformation exists (it is an optional field) */
	if (ppsOtherInformation != NULL)  {
		PDU_ptr->a.searchRequest->otherInfo = AsnListNew (sizeof(void *));
		CheckAsn1Alloc(PDU_ptr->a.searchRequest->otherInfo, env);

		DBVdbv_to_snacc_otherinfo(PDU_ptr->a.searchRequest->otherInfo, ppsOtherInformation, env);
	}
}






/*****************************************************************
******************************************************************
******************************************************************
PROCESS SearchResponse PDU RECEIVED FROM TARGET 
*/




int DBVset_dbv_fields_searchresp(DbvOriginData *psOriginData, PDU *PDU_ptr, jmp_buf env)
{

	psOriginData->eOriginEvent = OR_SearchConf;
	psOriginData->uOriginData.psSearchRsp = (DbvSearchRsp *) DbvAlloc (sizeof (DbvSearchRsp));
	CheckDbvAlloc(psOriginData->uOriginData.psSearchRsp, env);

	DBVset_dbv_searchresp_refid(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_rescount(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_numrecs(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_nextres(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_srchstat(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_resstat(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_presstat(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_records(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_addinfo(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);
	DBVset_dbv_searchresp_otherinfo(psOriginData->uOriginData.psSearchRsp, PDU_ptr, env);

	/* Following frees elements in PDU_ptr, not PDU_ptr itself */
	FreePDU(PDU_ptr);

}



int DBVset_dbv_searchresp_refid(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	/* If reference id given (it is an optional field) */
	if (PDU_ptr->a.searchResponse->referenceId.octetLen != 0){
		searchresp->psReferenceId = (DbvOctetString *) DbvAlloc (sizeof (DbvOctetString));
		CheckDbvAlloc(searchresp->psReferenceId, env);

		DBVsnacc_to_dbv_octet_string(searchresp->psReferenceId, &(PDU_ptr->a.searchResponse->referenceId), env); 
	}
}



int DBVset_dbv_searchresp_rescount(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	searchresp->ulResultCount = PDU_ptr->a.searchResponse->resultCount;
}



int DBVset_dbv_searchresp_numrecs(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	searchresp->ulNumberOfRecordsReturned = PDU_ptr->a.searchResponse->numberOfRecordsReturned;
}



int DBVset_dbv_searchresp_nextres(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	searchresp->ulNextResultSetPosition = PDU_ptr->a.searchResponse->nextResultSetPosition;
}



int DBVset_dbv_searchresp_srchstat(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->searchStatus == TRUE)
		searchresp->fSearchStatus = DBV_TRUE;
	else
		searchresp->fSearchStatus = DBV_FALSE;
}



int DBVset_dbv_searchresp_resstat(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->resultSetStatus != NULL) {
		searchresp->peResultSetStatus = (DbvResultSetStatus *) DbvAlloc (sizeof(DbvResultSetStatus));
		CheckDbvAlloc(searchresp->peResultSetStatus, env);

		switch (*PDU_ptr->a.searchResponse->resultSetStatus) {
		case SUBSET:
			*searchresp->peResultSetStatus = RSS_Subset;
			break;
		case INTERIM:
			*searchresp->peResultSetStatus = RSS_Interim;
			break;
		case NONE:
			*searchresp->peResultSetStatus = RSS_None;
			break;
		default:
			*searchresp->peResultSetStatus = RSS_Invalid;
			break;
		}
	}
}



int DBVset_dbv_searchresp_presstat(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->presentStatus != NULL) {
		searchresp->pePresentStatus = (DbvPresentStatus *) DbvAlloc (sizeof(DbvPresentStatus));
		CheckDbvAlloc(searchresp->pePresentStatus, env);

		/* In search, presentStatus is a pointer */
		DBVsnacc_to_dbv_presstat(searchresp->pePresentStatus, *PDU_ptr->a.searchResponse->presentStatus);
	}
}



int DBVset_dbv_searchresp_records(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	if (PDU_ptr->a.searchResponse->records != NULL) {
		searchresp->psRecords = (DbvRecords *) DbvAlloc (sizeof(struct DbvRecords));
		CheckDbvAlloc(searchresp->psRecords, env);

		DBVsnacc_to_dbv_records(searchresp->psRecords, PDU_ptr->a.searchResponse->records, env);
        }
}



int DBVset_dbv_searchresp_addinfo(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.searchResponse->additionalSearchInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.searchResponse->additionalSearchInfo);
		searchresp->ppsAdditionalSearchInfo = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(searchresp->ppsAdditionalSearchInfo, env);

		DBVsnacc_to_dbv_otherinfo(searchresp->ppsAdditionalSearchInfo, PDU_ptr->a.searchResponse->additionalSearchInfo, env);
        }
}



int DBVset_dbv_searchresp_otherinfo(DbvSearchRsp *searchresp, PDU *PDU_ptr, jmp_buf env)
{
	int num_elements = 0;

	/* If the otherinformation field is given (it is an optional field) */
	if (PDU_ptr->a.searchResponse->otherInfo != NULL) {
		num_elements = AsnListCount(PDU_ptr->a.searchResponse->otherInfo);
		searchresp->ppsOtherInformation = (DbvOtherInformation **) DbvAlloc (sizeof (DbvOtherInformation *) * (num_elements + 1));
		CheckDbvAlloc(searchresp->ppsOtherInformation, env);

		DBVsnacc_to_dbv_otherinfo(searchresp->ppsOtherInformation, PDU_ptr->a.searchResponse->otherInfo, env);
        }
}



